# -*- coding: utf-8 -*-
from operator import itemgetter

from wechatpy.utils import to_text
from wechatpy.client.api.base import BaseWeChatAPI


class WeChatTag(BaseWeChatAPI):
    def create(self, name):
        """
        创建标签

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :param name: 标签名（30个字符以内）
        :return: 返回的 JSON 数据包

        """
        name = to_text(name)
        return self._post(
            "tags/create",
            data={"tag": {"name": name}},
            result_processor=itemgetter("tag"),
        )

    def get(self):
        """
        获取公众号已创建的标签

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :return: 所有标签列表

        """
        return self._get("tags/get", result_processor=itemgetter("tags"))

    def update(self, tag_id, name):
        """
        编辑标签

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :param tag_id: 标签id，由微信分配
        :param name: 标签名字（30个字符以内）
        :return: 返回的 JSON 数据包

        """
        name = to_text(name)
        return self._post("tags/update", data={"tag": {"id": int(tag_id), "name": name}})

    def delete(self, tag_id):
        """
        删除标签

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :param tag_id: 标签 ID
        :return: 返回的 JSON 数据包

        """
        return self._post("tags/delete", data={"tag": {"id": tag_id}})

    def tag_user(self, tag_id, user_id):
        """
        批量为用户打标签

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :param tag_id: 标签 ID
        :param user_id: 用户 ID, 可以是单个或者列表

        :return: 返回的 JSON 数据包

        """
        data = {"tagid": tag_id}
        if isinstance(user_id, (tuple, list)):
            data["openid_list"] = user_id
        else:
            data["openid_list"] = [
                user_id,
            ]
        return self._post("tags/members/batchtagging", data=data)

    def untag_user(self, tag_id, user_id):
        """
        批量为用户取消标签

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :param tag_id: 标签 ID
        :param user_id: 用户 ID, 可以是单个或者列表

        :return: 返回的 JSON 数据包

        """
        data = {"tagid": tag_id}
        if isinstance(user_id, (tuple, list)):
            data["openid_list"] = user_id
        else:
            data["openid_list"] = [
                user_id,
            ]
        return self._post("tags/members/batchuntagging", data=data)

    def get_user_tag(self, user_id):
        """
        获取用户身上的标签列表

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :param user_id: 用户 ID, 可以是单个或者列表
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "tags/getidlist",
            data={"openid": user_id},
            result_processor=itemgetter("tagid_list"),
        )

    def get_tag_users(self, tag_id, first_user_id=None):
        """
        获取标签下粉丝列表

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html

        :param tag_id: 标签 ID
        :param first_user_id: 可选。第一个拉取的 OPENID，不填默认从头开始拉取
        :return: 返回的 JSON 数据包
        """
        data = {
            "tagid": tag_id,
        }
        if first_user_id:
            data["next_openid"] = first_user_id
        return self._post("user/tag/get", data=data)

    def iter_tag_users(self, tag_id, first_user_id=None):
        """
        获取标签下粉丝 openid 的生成器

        :return: 返回一个迭代器，可以用 for 进行循环，得到 openid

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    for openid in client.tag.iter_tag_users(0):
        >>>        print(openid)

        """
        while True:
            follower_data = self.get_tag_users(tag_id, first_user_id)
            if "data" not in follower_data:
                return
            for openid in follower_data["data"]["openid"]:
                yield openid
            first_user_id = follower_data.get("next_openid")
            if not first_user_id:
                return

    def get_black_list(self, begin_openid=None):
        """
        获取公众号的黑名单列表

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Manage_blacklist.html

        :param begin_openid: 起始的 OpenID，传空则默认从头开始拉取
        :return: 返回的 JSON 数据包
        :rtype: dict
        """
        data = {}
        if begin_openid:
            data["begin_openid"] = begin_openid
        return self._post(
            "tags/members/getblacklist",
            data=data,
        )

    def batch_black_list(self, openid_list):
        """
        批量拉黑用户

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Manage_blacklist.html

        :param openid_list: 批量拉黑用户的 OpenID list, 最多20个
        :type openid_list: list
        """
        return self._post(
            "tags/members/batchblacklist",
            data={
                "openid_list": openid_list,
            },
        )

    def batch_unblack_list(self, openid_list):
        """
        批量取消拉黑

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Manage_blacklist.html

        :param openid_list: 批量取消拉黑的 OpenID list, 最多20个
        :type openid_list: list
        """
        return self._post(
            "tags/members/batchunblacklist",
            data={
                "openid_list": openid_list,
            },
        )



# -*- coding: utf-8 -*-


class SessionStorage:
    def get(self, key, default=None):
        raise NotImplementedError()

    def set(self, key, value, ttl=None):
        raise NotImplementedError()

    def delete(self, key):
        raise NotImplementedError()

    def __getitem__(self, key):
        self.get(key)

    def __setitem__(self, key, value):
        self.set(key, value)

    def __delitem__(self, key):
        self.delete(key)



# -*- coding: utf-8 -*-
"""
    wechatpy.utils
    ~~~~~~~~~~~~~~~

    This module provides some useful utilities.

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""

import string
import random
import hashlib


class ObjectDict(dict):
    """Makes a dictionary behave like an object, with attribute-style access."""

    def __getattr__(self, key):
        if key in self:
            return self[key]
        return None

    def __setattr__(self, key, value):
        self[key] = value


class WeChatSigner:
    """WeChat data signer"""

    def __init__(self, delimiter=b""):
        self._data = []
        self._delimiter = to_binary(delimiter)

    def add_data(self, *args):
        """Add data to signer"""
        for data in args:
            self._data.append(to_binary(data))

    @property
    def signature(self):
        """Get data signature"""
        self._data.sort()
        str_to_sign = self._delimiter.join(self._data)
        return hashlib.sha1(str_to_sign).hexdigest()


def check_signature(token, signature, timestamp, nonce):
    """Check WeChat callback signature, raises InvalidSignatureException
    if check failed.

    :param token: WeChat callback token
    :param signature: WeChat callback signature sent by WeChat server
    :param timestamp: WeChat callback timestamp sent by WeChat server
    :param nonce: WeChat callback nonce sent by WeChat sever
    """
    signer = WeChatSigner()
    signer.add_data(token, timestamp, nonce)
    if signer.signature != signature:
        from wechatpy.exceptions import InvalidSignatureException

        raise InvalidSignatureException()


def check_wxa_signature(session_key, raw_data, client_signature):
    """校验前端传来的rawData签名正确
    详情请参考
    https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/signature.html # noqa

    :param session_key: code换取的session_key
    :param raw_data: 前端拿到的rawData
    :param client_signature: 前端拿到的signature
    :raises: InvalidSignatureException
    :return: 返回数据dict
    """
    str2sign = (raw_data + session_key).encode("utf-8")
    signature = hashlib.sha1(str2sign).hexdigest()
    if signature != client_signature:
        from wechatpy.exceptions import InvalidSignatureException

        raise InvalidSignatureException()


def to_text(value, encoding="utf-8"):
    """Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ""
    if isinstance(value, str):
        return value
    if isinstance(value, bytes):
        return value.decode(encoding)
    return str(value)


def to_binary(value, encoding="utf-8"):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b""
    if isinstance(value, bytes):
        return value
    if isinstance(value, str):
        return value.encode(encoding)
    return to_text(value).encode(encoding)


def timezone(zone):
    """Try to get timezone using pytz or python-dateutil

    :param zone: timezone str
    :return: timezone tzinfo or None
    """
    try:
        import pytz

        return pytz.timezone(zone)
    except ImportError:
        pass
    try:
        from dateutil.tz import gettz

        return gettz(zone)
    except ImportError:
        return None


def random_string(length=16):
    rule = string.ascii_letters + string.digits
    rand_list = random.sample(rule, length)
    return "".join(rand_list)



# -*- coding: utf-8 -*-

from urllib.parse import quote

import requests

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatQRCode(BaseWeChatAPI):
    def create(self, qrcode_data):
        """
        创建二维码

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Generating_a_Parametric_QR_Code.html

        :param qrcode_data: 你要发送的参数 dict
        :return: 返回的 JSON 数据包

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    res = client.qrcode.create({
        >>>        'expire_seconds': 1800,
        >>>        'action_name': 'QR_SCENE',
        >>>        'action_info': {
        >>>            'scene': {'scene_id': 123},
        >>>        }
        >>>    })
        >>>    # 创建永久的二维码, 参数使用字符串而不是数字id
        >>>    res = client.qrcode.create({
        >>>        'action_name': 'QR_LIMIT_STR_SCENE',
        >>>        'action_info': {
        >>>            'scene': {'scene_str': "scan_qrcode_from_scene"},
        >>>        }
        >>>    })

        """
        return self._post("qrcode/create", data=qrcode_data)

    def show(self, ticket):
        """
        通过ticket换取二维码

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Generating_a_Parametric_QR_Code.html

        :param ticket: 二维码 ticket 。可以通过 :func:`create` 获取到
        :return: 返回的 Request 对象

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    res = client.qrcode.show('ticket data')

        """
        if isinstance(ticket, dict):
            ticket = ticket["ticket"]
        return requests.get(url="https://mp.weixin.qq.com/cgi-bin/showqrcode", params={"ticket": ticket})

    @classmethod
    def get_url(cls, ticket):
        """
        通过ticket换取二维码地址

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Generating_a_Parametric_QR_Code.html

        :param ticket: 二维码 ticket 。可以通过 :func:`create` 获取到
        :return: 返回的二维码地址

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    url = client.qrcode.get_url('ticket data')

        """
        if isinstance(ticket, dict):
            ticket = ticket["ticket"]
        ticket = quote(ticket)
        return f"https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket={ticket}"



# -*- coding: utf-8 -*-

from optionaldict import optionaldict
from wechatpy.client.api.base import BaseWeChatAPI


class MerchantOrder(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def get(self, order_id):
        res = self._post(
            "merchant/order/getbyid",
            data={"order_id": order_id},
            result_processor=lambda x: x["order"],
        )
        return res

    def get_by_filter(self, status=None, begin_time=None, end_time=None):
        filter_dict = optionaldict(status=status, begintime=begin_time, endtime=end_time)

        res = self._post(
            "merchant/order/getbyfilter",
            data=dict(filter_dict),
            result_processor=lambda x: x["order_list"],
        )
        return res

    def set_delivery(self, order_id, company, track_no, need_delivery=1, is_others=0):
        return self._post(
            "merchant/order/setdelivery",
            data={
                "order_id": order_id,
                "delivery_company": company,
                "delivery_track_no": track_no,
                "need_delivery": need_delivery,
                "is_others": is_others,
            },
        )

    def close(self, order_id):
        return self._post("merchant/order/close", data={"order_id": order_id})



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wechatpy',
 'wechatpy.client',
 'wechatpy.client.api',
 'wechatpy.client.api.merchant',
 'wechatpy.crypto',
 'wechatpy.pay',
 'wechatpy.pay.api',
 'wechatpy.session',
 'wechatpy.work',
 'wechatpy.work.client',
 'wechatpy.work.client.api',
 'wechatpy.work.services',
 'wechatpy.work.services.api']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=3.1',
 'optionaldict>=0.1.0',
 'python-dateutil>=2.5.2',
 'requests-pkcs12>=1.7,<2.0',
 'requests>=2.4.3',
 'xmltodict>=0.11.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.7,<0.8']}

setup_kwargs = {
    'name': 'wechatpy',
    'version': '2.0.0a23',
    'description': 'WeChat SDK for Python',
    'long_description': '      ___       __   _______   ________  ___  ___  ________  _________  ________  ___    ___\n     |\\  \\     |\\  \\|\\  ___ \\ |\\   ____\\|\\  \\|\\  \\|\\   __  \\|\\___   ___\\\\   __  \\|\\  \\  /  /|\n     \\ \\  \\    \\ \\  \\ \\   __/|\\ \\  \\___|\\ \\  \\\\\\  \\ \\  \\|\\  \\|___ \\  \\_\\ \\  \\|\\  \\ \\  \\/  / /\n      \\ \\  \\  __\\ \\  \\ \\  \\_|/_\\ \\  \\    \\ \\   __  \\ \\   __  \\   \\ \\  \\ \\ \\   ____\\ \\    / /\n       \\ \\  \\|\\__\\_\\  \\ \\  \\_|\\ \\ \\  \\____\\ \\  \\ \\  \\ \\  \\ \\  \\   \\ \\  \\ \\ \\  \\___|\\/  /  /\n        \\ \\____________\\ \\_______\\ \\_______\\ \\__\\ \\__\\ \\__\\ \\__\\   \\ \\__\\ \\ \\__\\ __/  / /\n         \\|____________|\\|_______|\\|_______|\\|__|\\|__|\\|__|\\|__|    \\|__|  \\|__||\\___/ /\n                                                                                \\|___|/\n\n[![Financial Contributors on Open Collective](https://opencollective.com/wechatpy/all/badge.svg?label=financial+contributors)](https://opencollective.com/wechatpy) [![GitHub Actions](https://github.com/wechatpy/wechatpy/workflows/CI/badge.svg)](https://github.com/wechatpy/wechatpy/actions?query=workflow%3ACI)\n[![codecov.io](https://codecov.io/github/wechatpy/wechatpy/coverage.svg?branch=master)](https://codecov.io/github/wechatpy/wechatpy?branch=master)\n[![Documentation Status](https://readthedocs.org/projects/wechatpy/badge/?version=master)](http://docs.wechatpy.org/zh_CN/master/?badge=master)\n[![PyPI](https://img.shields.io/pypi/v/wechatpy.svg)](https://pypi.org/project/wechatpy)\n[![Downloads](https://pepy.tech/badge/wechatpy)](https://pepy.tech/project/wechatpy)\n[![Reviewed by Hound](https://img.shields.io/badge/Reviewed_by-Hound-8E64B0.svg)](https://houndci.com)\n\n微信(WeChat) 公众平台第三方 Python SDK。\n\nv1.x:   [【阅读文档】](https://wechatpy.readthedocs.org/zh_CN/stable/) [【快速入门】](https://wechatpy.readthedocs.org/zh_CN/stable/quickstart.html)\nmaster: [【阅读文档】](https://wechatpy.readthedocs.org/zh_CN/master/) [【快速入门】](https://wechatpy.readthedocs.org/zh_CN/master/quickstart.html)\n\n## 功能特性\n\n1. 普通公众平台被动响应和主动调用 API\n2. 企业微信 API\n3. 微信支付 API\n4. 第三方平台代公众号调用接口 API\n5. 小程序云开发 API\n\n## 安装\n\n推荐使用 pip 进行安装:\n\n```bash\npip install wechatpy\n```\n\n升级版本：\n\n    pip install -U wechatpy\n\n\n## 使用示例\n\n使用示例参见 [examples](examples/)\n\n## 贡献代码\n\n请阅读 [贡献代码指南](.github/CONTRIBUTING.md)\n\n## 支持项目\n\n如果觉得本项目对您有帮助，请考虑[捐赠支持项目开发](http://docs.wechatpy.org/zh_CN/master/sponsor.html)\n\n## 问题反馈\n\n我们主要使用 [GitHub issues](https://github.com/wechatpy/wechatpy/issues) 进行问题追踪和反馈。\n\nQQ 群：176596300\n\n![wechatpy QQ 群](https://raw.githubusercontent.com/wechatpy/wechatpy/master/docs/_static/images/qq-group.png)\n\n\n## Contributors\n\n### Code Contributors\n\nThis project exists thanks to all the people who contribute. [[Contribute](.github/CONTRIBUTING.md)].\n<a href="https://github.com/wechatpy/wechatpy/graphs/contributors"><img src="https://opencollective.com/wechatpy/contributors.svg?width=890&button=false" /></a>\n\n### Financial Contributors\n\nBecome a financial contributor and help us sustain our community. [[Contribute](https://opencollective.com/wechatpy/contribute)]\n\n#### Individuals\n\n<a href="https://opencollective.com/wechatpy"><img src="https://opencollective.com/wechatpy/individuals.svg?width=890"></a>\n\n#### Organizations\n\nSupport this project with your organization. Your logo will show up here with a link to your website. [[Contribute](https://opencollective.com/wechatpy/contribute)]\n\n<a href="https://opencollective.com/wechatpy/organization/0/website"><img src="https://opencollective.com/wechatpy/organization/0/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/1/website"><img src="https://opencollective.com/wechatpy/organization/1/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/2/website"><img src="https://opencollective.com/wechatpy/organization/2/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/3/website"><img src="https://opencollective.com/wechatpy/organization/3/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/4/website"><img src="https://opencollective.com/wechatpy/organization/4/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/5/website"><img src="https://opencollective.com/wechatpy/organization/5/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/6/website"><img src="https://opencollective.com/wechatpy/organization/6/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/7/website"><img src="https://opencollective.com/wechatpy/organization/7/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/8/website"><img src="https://opencollective.com/wechatpy/organization/8/avatar.svg"></a>\n<a href="https://opencollective.com/wechatpy/organization/9/website"><img src="https://opencollective.com/wechatpy/organization/9/avatar.svg"></a>\n\n## License\n\nThis work is released under the MIT license. A copy of the license is provided in the [LICENSE](./LICENSE) file.\n',
    'author': 'messense',
    'author_email': 'messense@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/wechatpy/wechatpy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
import json

from wechatpy.session import SessionStorage
from wechatpy.utils import to_text


class MemcachedStorage(SessionStorage):
    def __init__(self, mc, prefix="wechatpy"):
        for method_name in ("get", "set", "delete"):
            assert hasattr(mc, method_name)
        self.mc = mc
        self.prefix = prefix

    def key_name(self, key):
        return f"{self.prefix}:{key}"

    def get(self, key, default=None):
        key = self.key_name(key)
        value = self.mc.get(key)
        if value is None:
            return default
        return json.loads(to_text(value))

    def set(self, key, value, ttl=0):
        if value is None:
            return
        key = self.key_name(key)
        value = json.dumps(value)
        self.mc.set(key, value, ttl)

    def delete(self, key):
        key = self.key_name(key)
        self.mc.delete(key)



# -*- coding: utf-8 -*-

from wechatpy.utils import to_binary


class PKCS7Encoder:
    block_size = 32

    @classmethod
    def encode(cls, text):
        length = len(text)
        padding_count = cls.block_size - length % cls.block_size
        if padding_count == 0:
            padding_count = cls.block_size
        padding = to_binary(chr(padding_count))
        return text + padding * padding_count

    @classmethod
    def decode(cls, decrypted):
        padding = decrypted[-1]
        if padding < 1 or padding > 32:
            padding = 0
        return decrypted[:-padding]



# -*- coding: utf-8 -*-


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMedia(BaseWeChatAPI):
    """素材管理

    https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html
    """

    def upload(self, media_type, media_file):
        """
        新增临时素材
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html

        :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和缩略图（thumb）
        :param media_file: 要上传的文件，一个 File-object

        :return: 返回的 JSON 数据包
        """
        return self._post(url="media/upload", params={"type": media_type}, files={"media": media_file})

    def download(self, media_id):
        """
        获取临时素材
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Get_temporary_materials.html

        :param media_id: 媒体文件 ID

        :return: requests 的 Response 实例
        """
        return self._get("media/get", params={"media_id": media_id})

    def get_url(self, media_id):
        """
        获取临时素材下载地址

        :param media_id: 媒体文件 ID
        :return: 临时素材下载地址
        """
        return f"https://api.weixin.qq.com/cgi-bin/media/get?access_token={self.access_token}&media_id={media_id}"

    def upload_video(self, media_id, title, description):
        """
        群发视频消息时获取视频 media_id
        详情请参考
        http://mp.weixin.qq.com/wiki/15/5380a4e6f02f2ffdc7981a8ed7a40753.html

        :param media_id: 需通过基础支持中的上传下载多媒体文件 :func:`upload` 来得到
        :param title: 视频标题
        :param description: 视频描述

        :return: 返回的 JSON 数据包
        """
        return self._post(
            url="media/uploadvideo",
            data={"media_id": media_id, "title": title, "description": description},
        )

    def upload_articles(self, articles):
        """
        上传图文消息素材
        详情请参考
        http://mp.weixin.qq.com/wiki/15/5380a4e6f02f2ffdc7981a8ed7a40753.html

        :param articles: 图文消息数组
        :return: 返回的 JSON 数据包
        """
        articles_data = []
        for article in articles:
            articles_data.append(
                {
                    "thumb_media_id": article["thumb_media_id"],
                    "title": article["title"],
                    "content": article["content"],
                    "author": article.get("author", ""),
                    "content_source_url": article.get("content_source_url", ""),
                    "digest": article.get("digest", ""),
                    "show_cover_pic": article.get("show_cover_pic", 0),
                }
            )
        return self._post("media/uploadnews", data={"articles": articles_data})

    def upload_image(self, media_file):
        """
        上传群发消息内的图片
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Adding_Permanent_Assets.html

        :param media_file: 要上传的文件，一个 File-object
        :return: 上传成功时返回图片 URL
        """
        res = self._post(
            url="media/uploadimg",
            files={"media": media_file},
            result_processor=lambda x: x["url"],
        )
        return res

    upload_mass_image = upload_image



# -*- coding: utf-8 -*-
import json
import time
import inspect
import logging

import requests

from wechatpy.constants import WeChatErrorCode
from wechatpy.session.memorystorage import MemoryStorage
from wechatpy.exceptions import WeChatClientException, APILimitedException
from wechatpy.client.api.base import BaseWeChatAPI


logger = logging.getLogger(__name__)


def _is_api_endpoint(obj):
    return isinstance(obj, BaseWeChatAPI)


class BaseWeChatClient:
    API_BASE_URL = ""

    def __new__(cls, *args, **kwargs):
        self = super().__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self)
            setattr(self, name, api)
        return self

    def __init__(self, appid, access_token=None, session=None, timeout=None, auto_retry=True):
        self._http = requests.Session()
        self.appid = appid
        self.session = session or MemoryStorage()
        self.timeout = timeout
        self.auto_retry = auto_retry

        if access_token:
            self.session.set(self.access_token_key, access_token)

    @property
    def access_token_key(self):
        return f"{self.appid}_access_token"

    @property
    def access_token_expires_at_key(self):
        return f"{self.appid}_access_token_expires_at"

    @property
    def expires_at(self):
        return self.session.get(self.access_token_expires_at_key, None)

    @expires_at.setter
    def expires_at(self, value):
        self.session.set(self.access_token_expires_at_key, value)

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(("http://", "https://")):
            api_base_url = kwargs.pop("api_base_url", self.API_BASE_URL)
            url = f"{api_base_url}{url_or_endpoint}"
        else:
            url = url_or_endpoint

        if "params" not in kwargs:
            kwargs["params"] = {}
        if isinstance(kwargs["params"], dict) and "access_token" not in kwargs["params"]:
            kwargs["params"]["access_token"] = self.access_token
        if isinstance(kwargs.get("data", ""), dict):
            body = json.dumps(kwargs["data"], ensure_ascii=False)
            body = body.encode("utf-8")
            kwargs["data"] = body

        kwargs["timeout"] = kwargs.get("timeout", self.timeout)
        result_processor = kwargs.pop("result_processor", None)
        res = self._http.request(method=method, url=url, **kwargs)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )

        return self._handle_result(res, method, url, result_processor, **kwargs)

    def _decode_result(self, res):
        try:
            result = json.loads(res.content.decode("utf-8", "ignore"), strict=False)
        except (TypeError, ValueError):
            # Return origin response object if we can not decode it as JSON
            logger.debug("Can not decode response as JSON", exc_info=True)
            return res
        return result

    def _handle_result(self, res, method=None, url=None, result_processor=None, **kwargs):
        if not isinstance(res, dict):
            # Dirty hack around asyncio based AsyncWeChatClient
            result = self._decode_result(res)
        else:
            result = res

        if not isinstance(result, dict):
            return result

        if "base_resp" in result:
            # Different response in device APIs. Fuck tencent!
            result.update(result.pop("base_resp"))
        if "errcode" in result:
            result["errcode"] = int(result["errcode"])

        if "errcode" in result and result["errcode"] != 0:
            errcode = result["errcode"]
            errmsg = result.get("errmsg", errcode)
            if self.auto_retry and errcode in (
                WeChatErrorCode.INVALID_CREDENTIAL.value,
                WeChatErrorCode.INVALID_ACCESS_TOKEN.value,
                WeChatErrorCode.EXPIRED_ACCESS_TOKEN.value,
            ):
                logger.info("Access token expired, fetch a new one and retry request")
                self.fetch_access_token()
                access_token = self.session.get(self.access_token_key)
                kwargs["params"]["access_token"] = access_token
                return self._request(method=method, url_or_endpoint=url, result_processor=result_processor, **kwargs)
            elif errcode == WeChatErrorCode.OUT_OF_API_FREQ_LIMIT.value:
                # api freq out of limit
                raise APILimitedException(errcode, errmsg, client=self, request=res.request, response=res)
            else:
                raise WeChatClientException(errcode, errmsg, client=self, request=res.request, response=res)

        return result if not result_processor else result_processor(result)

    def get(self, url, **kwargs):
        return self._request(method="get", url_or_endpoint=url, **kwargs)

    def post(self, url, **kwargs):
        return self._request(method="post", url_or_endpoint=url, **kwargs)

    def _fetch_access_token(self, url, params):
        """The real fetch access token"""
        logger.info("Fetching access token")
        res = self._http.get(url=url, params=params)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )
        result = res.json()
        if "errcode" in result and result["errcode"] != 0:
            raise WeChatClientException(
                result["errcode"],
                result["errmsg"],
                client=self,
                request=res.request,
                response=res,
            )

        expires_in = 7200
        if "expires_in" in result:
            expires_in = result["expires_in"]
        self.session.set(self.access_token_key, result["access_token"], expires_in)
        self.expires_at = int(time.time()) + expires_in
        return result

    def fetch_access_token(self):
        raise NotImplementedError()

    @property
    def access_token(self):
        """WeChat access token"""
        access_token = self.session.get(self.access_token_key)
        if access_token:
            if not self.expires_at:
                # user provided access_token, just return it
                return access_token

            timestamp = time.time()
            if self.expires_at - timestamp > 60:
                return access_token

        self.fetch_access_token()
        return self.session.get(self.access_token_key)



# -*- coding: utf-8 -*-
from urllib.parse import quote
from typing import Any, Dict

from wechatpy.pay import calculate_signature_hmac
from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatAppAuth(BaseWeChatPayAPI):
    """微信支付实名认证接口"""

    def get_auth_url(self, redirect_uri: str, state: str) -> str:
        """
        获取实名认证授权重定向地址，如果用户同意授权，页面将跳转至 `redirect_uri/?code=CODE&state=STATE`；
        code 作为换取 access_token 的票据，每次用户授权带上的 code 将不一样，code 只能使用一次，10分钟未被使用自动过期。

        详情请参考 https://pay.weixin.qq.com/wiki/doc/api/realnameauth.php?chapter=60_1&index=2

        :param redirect_uri: 重定向地址，自动 urlencode，需在支付安全域下(商户平台上配置“支付授权目录”)
        :param state: 随机字符串，回调时将带上该参数
        """
        redirect_uri = quote(redirect_uri, safe=b"")
        url_parts = [
            "https://payapp.weixin.qq.com/appauth/authindex?mch_id=",
            self.mch_id,
            "&appid=",
            self.appid,
            "&redirect_uri=",
            redirect_uri,
            "&response_type=code&scope=pay_identity&state=",
            state,
            "#wechat_redirect",
        ]
        return "".join(url_parts)

    def get_access_token(self, openid: str, code: str) -> Dict[str, Any]:
        """
        获取微信用户的授权, 用授权小程序得到的授权码调用 OAuth2.0 接口 access_token

        :param openid: 用户 openid
        :param code: 预授权码
        """
        params = {
            "mch_id": self.mch_id,
            "appid": self.appid,
            "openid": openid,
            "code": code,
            "scope": "pay_identity",
            "grant_type": "authorization_code",
            "sign_type": "HMAC-SHA256",
        }
        sign = calculate_signature_hmac(params, self._client.api_key)
        params["sign"] = sign
        res = self._get(
            "appauth/getaccesstoken",
            params=params,
        )
        return res

    def real_name_auth(self, openid: str, real_name: str, cred_id: str, access_token: str) -> Dict[str, Any]:
        """
        取得 access_token 后调用本接口验证微信用户的姓名和身份证信息是否匹配

        :param openid: 用户 openid
        :param real_name: 真实姓名
        :param cred_id: 身份证号码
        :param access_token: 获取用户授权后换取的 access_token
        """
        return self._post(
            "https://fraud.mch.weixin.qq.com/secsvc/realnameauth",
            data={
                "version": "1.0",
                "appid": self.appid,
                "openid": openid,
                "real_name": real_name,
                "cred_type": 1,
                "cred_id": cred_id,
                "access_token": access_token,
                "sign_type": "HMAC-SHA256",
            },
        )



# -*- coding: utf-8 -*-
"""
    wechatpy.messages
    ~~~~~~~~~~~~~~~~~~

    This module defines all the messages you can get from WeChat server

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
import copy

from wechatpy.fields import BaseField, DateTimeField, FieldDescriptor, IntegerField, StringField

MESSAGE_TYPES = {}
COMPONENT_MESSAGE_TYPES = {}


def register_message(msg_type):
    def register(cls):
        MESSAGE_TYPES[msg_type] = cls
        return cls

    return register


def register_component_message(msg_type):
    def register(cls):
        COMPONENT_MESSAGE_TYPES[msg_type] = cls
        return cls

    return register


class MessageMetaClass(type):
    """Metaclass for all messages"""

    def __new__(mcs, name, bases, attrs):
        for b in bases:
            if not hasattr(b, "_fields"):
                continue

            for k, v in b.__dict__.items():
                if k in attrs:
                    continue
                if isinstance(v, FieldDescriptor):
                    attrs[k] = copy.deepcopy(v.field)

        mcs = super().__new__(mcs, name, bases, attrs)
        mcs._fields = {}

        for name, field in mcs.__dict__.items():
            if isinstance(field, BaseField):
                field.add_to_class(mcs, name)
        return mcs


class BaseMessage(metaclass=MessageMetaClass):
    """Base class for all messages and events"""

    type = "unknown"
    id = IntegerField("MsgId", 0)
    source = StringField("FromUserName")
    target = StringField("ToUserName")
    create_time = DateTimeField("CreateTime")
    time = IntegerField("CreateTime")

    def __init__(self, message):
        self._data = message

    def __repr__(self):
        return f"{self.__class__.__name__}({repr(self._data)})"


@register_message("text")
class TextMessage(BaseMessage):
    """
    文本消息
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_standard_messages.html
    """

    type = "text"
    content = StringField("Content")


@register_message("image")
class ImageMessage(BaseMessage):
    """
    图片消息
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_standard_messages.html
    """

    type = "image"
    media_id = StringField("MediaId")
    image = StringField("PicUrl")


@register_message("voice")
class VoiceMessage(BaseMessage):
    """
    语音消息
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_standard_messages.html
    """

    type = "voice"
    media_id = StringField("MediaId")
    format = StringField("Format")
    recognition = StringField("Recognition")


@register_message("shortvideo")
class ShortVideoMessage(BaseMessage):
    """
    短视频消息
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_standard_messages.html
    """

    type = "shortvideo"
    media_id = StringField("MediaId")
    thumb_media_id = StringField("ThumbMediaId")


@register_message("video")
class VideoMessage(BaseMessage):
    """
    视频消息
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_standard_messages.html
    """

    type = "video"
    media_id = StringField("MediaId")
    thumb_media_id = StringField("ThumbMediaId")


@register_message("location")
class LocationMessage(BaseMessage):
    """
    地理位置消息
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_standard_messages.html
    """

    type = "location"
    location_x = StringField("Location_X")
    location_y = StringField("Location_Y")
    scale = StringField("Scale")
    label = StringField("Label")

    @property
    def location(self):
        return self.location_x, self.location_y


@register_message("link")
class LinkMessage(BaseMessage):
    """
    链接消息
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_standard_messages.html
    """

    type = "link"
    title = StringField("Title")
    description = StringField("Description")
    url = StringField("Url")


@register_message("miniprogrampage")
class MiniProgramPageMessage(BaseMessage):
    """
    小程序卡片消息
    详情请参阅
    https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/customer-message/receive.html#小程序卡片消息
    """

    type = "miniprogrampage"
    app_id = StringField("AppId")
    title = StringField("Title")
    page_path = StringField("PagePath")
    thumb_url = StringField("ThumbUrl")
    thumb_media_id = StringField("ThumbMediaId")


class UnknownMessage(BaseMessage):
    """未知消息类型"""

    pass


class BaseComponentMessage(metaclass=MessageMetaClass):
    """Base class for all component messages and events"""

    type = "unknown"
    appid = StringField("AppId")
    create_time = DateTimeField("CreateTime")

    def __init__(self, message):
        self._data = message

    def __repr__(self):
        return f"{self.__class__.__name__}({repr(self._data)})"


@register_component_message("component_verify_ticket")
class ComponentVerifyTicketMessage(BaseComponentMessage):
    """
    component_verify_ticket协议
    """

    type = "component_verify_ticket"
    verify_ticket = StringField("ComponentVerifyTicket")


@register_component_message("unauthorized")
class ComponentUnauthorizedMessage(BaseComponentMessage):
    """
    取消授权通知
    """

    type = "unauthorized"
    authorizer_appid = StringField("AuthorizerAppid")


@register_component_message("authorized")
class ComponentAuthorizedMessage(BaseComponentMessage):
    """
    新增授权通知
    """

    type = "authorized"
    authorizer_appid = StringField("AuthorizerAppid")
    authorization_code = StringField("AuthorizationCode")
    authorization_code_expired_time = StringField("AuthorizationCodeExpiredTime")
    pre_auth_code = StringField("PreAuthCode")


@register_component_message("updateauthorized")
class ComponentUpdateAuthorizedMessage(BaseComponentMessage):
    """
    更新授权通知
    """

    type = "updateauthorized"
    authorizer_appid = StringField("AuthorizerAppid")
    authorization_code = StringField("AuthorizationCode")
    authorization_code_expired_time = StringField("AuthorizationCodeExpiredTime")
    pre_auth_code = StringField("PreAuthCode")


class ComponentUnknownMessage(BaseComponentMessage):
    """
    未知通知
    """

    type = "unknown"



# -*- coding: utf-8 -*-

import json
from enum import IntEnum

import requests

from wechatpy.client.api.base import BaseWeChatAPI


class FileType(IntEnum):
    JSON = 1
    CSV = 2


class ConflictMode(IntEnum):
    INSERT = 1
    UPSERT = 2


class WeChatCloud(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/"

    def invoke_cloud_function(self, env, name, data=None):
        """
        触发云函数

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/functions/invokeCloudFunction.html

        :param env: 云开发环境 ID
        :param name: 云函数名称
        :param data: 云函数的传入参数，具体结构由开发者定义
        """
        return self._post(
            "tcb/invokecloudfunction",
            params={
                "env": env,
                "name": name,
            },
            data=data,
            result_processor=lambda x: json.loads(x["resp_data"]),
        )

    def db_import(self, env, collection, file_path, file_type, conflict_mode, stop_on_error=True):
        """
        数据库导入

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseMigrateImport.html

        :param env: 云开发环境 ID
        :param collection: 导入 collection 名称
        :param file_path: 导入文件路径(导入文件需先上传到同环境的存储中，可使用开发者工具或 HTTP API的上传文件 API上传）
        :param file_type: 导入文件类型，文件格式参考数据库导入指引中的文件格式部分，值为数字，1 为 JSON，2 为 CSV
        :param stop_on_error: 是否在遇到错误时停止导入，默认为 True
        :param conflict_mode: 冲突处理模式, 值为数字，1 为 INSERT，2 为 UPSERT
        :return: 导入任务 ID，可使用数据库迁移进度查询 API 查询导入进度及结果
        """
        return self._post(
            "tcb/databasemigrateimport",
            data={
                "env": env,
                "collection_name": collection,
                "file_path": file_path,
                "file_type": file_type,
                "stop_on_error": stop_on_error,
                "conflict_mode": conflict_mode,
            },
            result_processor=lambda x: x["job_id"],
        )

    def db_export(self, env, file_path, file_type, query):
        """
        数据库导出

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseMigrateExport.html

        :param env: 云开发环境 ID
        :param file_path: 导出文件路径(导入文件需先上传到同环境的存储中，可使用开发者工具或 HTTP API的上传文件 API上传）
        :param file_type: 导出文件类型，文件格式参考数据库导入指引中的文件格式部分，值为数字，1 为 JSON，2 为 CSV
        :param query: 导出条件
        :return: 导出任务 ID，可使用数据库迁移进度查询 API 查询导出结果，获取文件下载链接
        """
        return self._post(
            "tcb/databasemigrateexport",
            data={
                "env": env,
                "file_path": file_path,
                "file_type": file_type,
                "query": query,
            },
            result_processor=lambda x: x["job_id"],
        )

    def db_query_migrate_info(self, env, job_id):
        """
        数据库迁移状态查询

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseMigrateQueryInfo.html

        :param env: 云开发环境 ID
        :param job_id: 任务 ID
        """
        return self._post(
            "tcb/databasemigratequeryinfo",
            data={
                "env": env,
                "job_id": job_id,
            },
        )

    def db_update_index(self, env, collection, create_indexes=None, drop_indexes=None):
        """
        变更数据库索引

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/updateIndex.html

        :param env: 云开发环境 ID
        :param collection: 导出 collection 名称
        """
        assert create_indexes or drop_indexes
        return self._post(
            "tcb/updateindex",
            data={
                "env": env,
                "collection_name": collection,
                "create_indexes": create_indexes or [],
                "drop_indexes": create_indexes or [],
            },
        )

    def db_add_collection(self, env, collection):
        """
        新增集合

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseCollectionAdd.html

        :param env: 云开发环境 ID
        :param collection: 集合名称
        """
        return self._post(
            "tcb/databasecollectionadd",
            data={
                "env": env,
                "collection_name": collection,
            },
        )

    def db_delete_collection(self, env, collection):
        """
        删除集合

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseCollectionDelete.html

        :param env: 云开发环境 ID
        :param collection: 集合名称
        """
        return self._post(
            "tcb/databasecollectiondelete",
            data={
                "env": env,
                "collection_name": collection,
            },
        )

    def db_get_collection(self, env, offset=0, limit=10):
        """
        获取特定云环境下集合信息

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseCollectionGet.html

        :param env: 云开发环境 ID
        :param offset: 偏移量，默认为 0
        :param limit: 获取数量限制， 默认为 10
        """
        return self._post(
            "tcb/databasecollectionget",
            data={
                "env": env,
                "offset": offset,
                "limit": limit,
            },
        )

    def db_add(self, env, query):
        """
        数据库插入记录

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseAdd.html

        :param env: 云开发环境 ID
        :param query: 数据库操作语句
        :return: 返回插入成功的数据集合主键 _id 列表
        """
        return self._post(
            "tcb/databaseadd",
            data={
                "env": env,
                "query": query,
            },
            result_processor=lambda x: x["id_list"],
        )

    def db_delete(self, env, query):
        """
        数据库删除记录

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseDelete.html

        :param env: 云开发环境 ID
        :param query: 数据库操作语句
        :return: 返回删除记录数量
        """
        return self._post(
            "tcb/databasedelete",
            data={
                "env": env,
                "query": query,
            },
            result_processor=lambda x: x["deleted"],
        )

    def db_update(self, env, query):
        """
        数据更新记录

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseUpdate.html

        :param env: 云开发环境 ID
        :param query: 数据库操作语句
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "tcb/databaseupdate",
            data={
                "env": env,
                "query": query,
            },
        )

    def db_query(self, env, query):
        """
        数据库查询记录

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseQuery.html

        :param env: 云开发环境 ID
        :param query: 数据库操作语句
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "tcb/databasequery",
            data={
                "env": env,
                "query": query,
            },
        )

    def db_aggregate(self, env, query):
        """
        数据库聚合

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseAggregate.html

        :param env: 云开发环境 ID
        :param query: 数据库操作语句
        :return: 返回记录列表
        """
        return self._post(
            "tcb/databaseaggregate",
            data={
                "env": env,
                "query": query,
            },
            result_processor=lambda x: x["data"],
        )

    def db_count(self, env, query):
        """
        统计集合记录数或统计查询语句对应的结果记录数

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/database/databaseCount.html

        :param env: 云开发环境 ID
        :param query: 数据库操作语句
        :return: 返回记录数量
        """
        return self._post(
            "tcb/databasecount",
            data={
                "env": env,
                "query": query,
            },
            result_processor=lambda x: x["count"],
        )

    def upload_file(self, env, path):
        """
        获取文件上传链接

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/storage/uploadFile.html

        :param env: 云开发环境 ID
        """
        with open(path, "rb") as f:
            res = self._post(
                "tcb/uploadfile",
                data={
                    "env": env,
                    "path": path,
                },
            )
            signature = res["authorization"]
            token = res["token"]
            cos_file_id = res["cos_file_id"]
            upload_res = requests.post(
                res["url"],
                files={
                    "key": path,
                    "Signature": signature,
                    "x-cos-security-token": token,
                    "x-cos-meta-fileid": cos_file_id,
                    # 注意！file 字段须放在最后，否则上传大文件会失败
                    "file": f,
                },
            )
            upload_res.raise_for_status()
            return upload_res

    def download_files(self, env, file_list):
        """
        获取文件下载链接

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/storage/batchDownloadFile.html

        :param env: 云开发环境 ID
        :param file_list: 文件列表
        :return: 返回文件列表
        """
        return self._post(
            "tcb/batchdownloadfile",
            data={
                "env": env,
                "file_list": file_list,
            },
            result_processor=lambda x: x["file_list"],
        )

    def delete_files(self, env, fileid_list):
        """
        删除文件

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/storage/batchDeleteFile.html

        :param env: 云开发环境 ID
        :param fileid_list: 文件 ID 列表
        :return: 被删除的文件列表
        """
        return self._post(
            "tcb/batchdeletefile",
            data={
                "env": env,
                "fileid_list": fileid_list,
            },
            result_processor=lambda x: x["delete_list"],
        )

    def get_qcloud_token(self, lifespan=7200):
        """
        获取腾讯云 API 调用凭证

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/wxcloud/reference-http-api/utils/getQcloudToken.html
        """
        return self._post("tcb/getqcloudtoken", data={"lifespan": lifespan})



# -*- coding: utf-8 -*-
"""
    wechatpy.events
    ~~~~~~~~~~~~~~~~

    This module contains all the events WeChat callback uses.

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
from typing import Dict, List

from wechatpy.fields import (
    Base64DecodeField,
    BaseField,
    DateTimeField,
    FloatField,
    IntegerField,
    StringField,
)
from wechatpy.messages import BaseMessage

EVENT_TYPES = {}


class BaseEvent(BaseMessage):
    """Base class for all events"""

    type = "event"
    event = ""

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        # 注册子类
        if cls.event:
            EVENT_TYPES[cls.event] = cls


class SubscribeEvent(BaseEvent):
    """
    用户关注事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
    """

    event = "subscribe"
    key = StringField("EventKey", "")


class UnsubscribeEvent(BaseEvent):
    """
    用户取消关注事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
    """

    event = "unsubscribe"


class SubscribeScanEvent(BaseEvent):
    """
    用户扫描二维码关注事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
    """

    event = "subscribe_scan"
    scene_id = StringField("EventKey")
    ticket = StringField("Ticket")


class ScanEvent(BaseEvent):
    """
    用户扫描二维码事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
    """

    event = "scan"
    scene_id = StringField("EventKey")
    ticket = StringField("Ticket")


class LocationEvent(BaseEvent):
    """
    上报地理位置事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
    """

    event = "location"
    latitude = FloatField("Latitude", 0.0)
    longitude = FloatField("Longitude", 0.0)
    precision = FloatField("Precision", 0.0)


class ClickEvent(BaseEvent):
    """
    点击菜单拉取消息事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
    """

    event = "click"
    key = StringField("EventKey")


class ViewEvent(BaseEvent):
    """
    点击菜单跳转链接事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
    """

    event = "view"
    url = StringField("EventKey")


class MassSendJobFinishEvent(BaseEvent):
    """
    群发消息任务完成事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Batch_Sends_and_Originality_Checks.html#7
    """

    id = IntegerField("MsgID", 0)
    event = "masssendjobfinish"
    status = StringField("Status")
    total_count = IntegerField("TotalCount", 0)
    filter_count = IntegerField("FilterCount", 0)
    sent_count = IntegerField("SentCount", 0)
    error_count = IntegerField("ErrorCount", 0)


class TemplateSendJobFinishEvent(BaseEvent):
    """
    模板消息任务完成事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#6
    """

    id = IntegerField("MsgID")
    event = "templatesendjobfinish"
    status = StringField("Status")


class SubscribeMsgPopupEvent(BaseEvent):
    """
    用户操作订阅通知弹窗事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html
    """

    event = "subscribe_msg_popup_event"
    subscribes_info = BaseField("SubscribeMsgPopupEvent", {})

    @property
    def subscribes(self) -> List[Dict]:
        """
        返回值参考:
        [
          {
            "TemplateId": "VRR0UEO9VJOLs0MHlU0OilqX6MVFDwH3_3gz3Oc0NIc",
            "SubscribeStatusString": "accept",
            "PopupScene": 2
          },
          {
            "TemplateId": "9nLIlbOQZC5Y89AZteFEux3WCXRRRG5Wfzkpssu4bLI",
            "SubscribeStatusString": "reject",
            "PopupScene": 2
          },
        ]
        """
        subscribes = self.subscribes_info["List"]
        if not isinstance(subscribes, list):
            subscribes = [subscribes]
        return subscribes


class SubscribeMsgChangeEvent(BaseEvent):
    """
    用户管理订阅通知事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html
    """

    event = "subscribe_msg_change_event"
    subscribes_info = BaseField("SubscribeMsgChangeEvent", {})

    @property
    def subscribes(self) -> List[Dict]:
        """
        返回值参考:
        [
          {
            "TemplateId": "VRR0UEO9VJOLs0MHlU0OilqX6MVFDwH3_3gz3Oc0NIc",
            "SubscribeStatusString": "accept",
          },
          {
            "TemplateId": "9nLIlbOQZC5Y89AZteFEux3WCXRRRG5Wfzkpssu4bLI",
            "SubscribeStatusString": "reject",
          },
        ]
        """
        subscribes = self.subscribes_info["List"]
        if not isinstance(subscribes, list):
            subscribes = [subscribes]
        return subscribes


class SubscribeMsgSentEvent(BaseEvent):
    """
    发送订阅通知事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html
    """

    event = "subscribe_msg_sent_event"
    subscribes_info = BaseField("SubscribeMsgSentEvent", {})

    @property
    def subscribes(self) -> List[Dict]:
        """
        返回值参考:
        [
          {
            "TemplateId": "VRR0UEO9VJOLs0MHlU0OilqX6MVFDwH3_3gz3Oc0NIc",
            "MsgID": "1700827132819554304",
            "ErrorCode": "0",
            "ErrorStatus": "success",
          },
        ]
        """
        subscribes = self.subscribes_info["List"]
        if not isinstance(subscribes, list):
            subscribes = [subscribes]
        return subscribes


class BaseScanCodeEvent(BaseEvent):
    key = StringField("EventKey")
    scan_code_info = BaseField("ScanCodeInfo", {})

    @property
    def scan_type(self):
        return self.scan_code_info["ScanType"]

    @property
    def scan_result(self):
        return self.scan_code_info["ScanResult"]


class ScanCodePushEvent(BaseScanCodeEvent):
    """
    扫码推事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Custom_Menu_Push_Events.html
    """

    event = "scancode_push"


class ScanCodeWaitMsgEvent(BaseScanCodeEvent):
    """
    扫码推事件且弹出“消息接收中”提示框的事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Custom_Menu_Push_Events.html
    """

    event = "scancode_waitmsg"


class BasePictureEvent(BaseEvent):
    key = StringField("EventKey")
    pictures_info = BaseField("SendPicsInfo", {})

    @property
    def count(self):
        return int(self.pictures_info["Count"])

    @property
    def pictures(self):
        if self.pictures_info["PicList"]:
            items = self.pictures_info["PicList"]["item"]
            if self.count > 1:
                return items
            return [items]
        return []


class PicSysPhotoEvent(BasePictureEvent):
    """
    弹出系统拍照发图的事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Custom_Menu_Push_Events.html
    """

    event = "pic_sysphoto"


class PicPhotoOrAlbumEvent(BasePictureEvent):
    """
    弹出拍照或者相册发图的事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Custom_Menu_Push_Events.html
    """

    event = "pic_photo_or_album"


class PicWeChatEvent(BasePictureEvent):
    """
    弹出微信相册发图器的事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Custom_Menu_Push_Events.html
    """

    event = "pic_weixin"


class LocationSelectEvent(BaseEvent):
    """
    弹出地理位置选择器的事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Custom_Menu_Push_Events.html
    """

    event = "location_select"
    key = StringField("EventKey")
    location_info = BaseField("SendLocationInfo", {})

    @property
    def location_x(self):
        return self.location_info["Location_X"]

    @property
    def location_y(self):
        return self.location_info["Location_Y"]

    @property
    def location(self):
        return self.location_x, self.location_y

    @property
    def scale(self):
        return self.location_info["Scale"]

    @property
    def label(self):
        return self.location_info["Label"]

    @property
    def poiname(self):
        return self.location_info["Poiname"]


class CardPassCheckEvent(BaseEvent):
    """
    卡券审核事件推送

    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#1
    """

    event = "card_pass_check"
    card_id = StringField("CardId")


class CardNotPassCheckEvent(BaseEvent):
    event = "card_not_pass_check"
    card_id = StringField("CardId")
    refuse_reason = StringField("RefuseReason")


class UserGetCardEvent(BaseEvent):
    """
    领取事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#2
    """

    event = "user_get_card"
    card_id = StringField("CardId")
    is_given_by_friend = IntegerField("IsGiveByFriend")
    friend = StringField("FriendUserName")
    code = StringField("UserCardCode")
    old_code = StringField("OldUserCardCode")
    outer_id = StringField("OuterId")
    outer_str = StringField("OuterStr")
    is_restore_member_card = IntegerField("IsRestoreMemberCard")
    is_recommend_by_friend = IntegerField("IsRecommendByFriend")
    union_id = StringField("UnionId")


class UserGiftingCardEvent(BaseEvent):
    """
    转赠事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#3
    """

    event = "user_gifting_card"
    card_id = StringField("CardId")
    code = StringField("UserCardCode")
    is_return_back = IntegerField("IsReturnBack")
    friend = StringField("FriendUserName")
    is_chat_room = IntegerField("IsChatRoom")


class UserDeleteCardEvent(BaseEvent):
    """
    卡券删除事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#4
    """

    event = "user_del_card"
    card_id = StringField("CardId")
    code = StringField("UserCardCode")


class UserConsumeCardEvent(BaseEvent):
    """
    卡券核销事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#5
    """

    event = "user_consume_card"
    card_id = StringField("CardId")
    code = StringField("UserCardCode")
    consume_source = StringField("ConsumeSource")
    location_name = StringField("LocationName")
    staff = StringField("StaffOpenId")
    verify_code = StringField("VerifyCode")
    remark_amount = StringField("RemarkAmount")
    outer_str = StringField("OuterStr")


class UserPayFromPayCell(BaseEvent):
    """
    卡券买单事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#6
    """

    event = "user_pay_from_pay_cell"
    card_id = StringField("CardId")
    code = StringField("UserCardCode")
    trans_id = StringField("TransId")
    location_id = IntegerField("LocationId")
    fee = IntegerField("Fee")
    original_fee = IntegerField("OriginalFee")


class UserViewCard(BaseEvent):
    """
    进入会员卡事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#7
    """

    event = "user_view_card"
    card_id = StringField("CardId")
    code = StringField("UserCardCode")
    outer_str = StringField("OuterStr")


class UserEnterSessionFromCardEvent(BaseEvent):
    """
    从卡券进入公众号会话事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#8
    """

    event = "user_enter_session_from_card"
    card_id = StringField("CardId")
    code = StringField("UserCardCode")


class UpdateMemberCard(BaseEvent):
    """
    会员卡内容更新事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#9
    """

    event = "update_member_card"
    card_id = StringField("CardId")
    code = StringField("UserCardCode")
    modify_bonus = IntegerField("ModifyBonus")
    modify_balance = IntegerField("ModifyBalance")


class CardSkuRemindEvent(BaseEvent):
    """
    卡券库存报警事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#10
    """

    event = "card_sku_remind"
    card_id = StringField("CardId")
    detail = StringField("Detail")


class CardPayOrderEvent(BaseEvent):
    """
    券点流水详情事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#11
    """

    event = "card_pay_order"
    order_id = IntegerField("OrderId")
    status = StringField("Status")
    create_order_time = IntegerField("CreateOrderTime")
    pay_finish_time = IntegerField("PayFinishTime")
    description = StringField("Desc")
    free_coin_count = IntegerField("FreeCoinCount")
    pay_coin_count = IntegerField("PayCoinCount")
    refund_free_coin_count = IntegerField("RefundFreeCoinCount")
    refund_pay_coin_count = IntegerField("RefundPayCoinCount")
    order_type = StringField("OrderType")
    memo = StringField("Memo")
    receipt_info = StringField("ReceiptInfo")


class SubmitMembercardUserInfo(BaseEvent):
    """
    会员卡激活事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#12
    """

    event = "submit_membercard_user_info"
    card_id = StringField("CardId")
    card_code = StringField("UserCardCode")


class MerchantOrderEvent(BaseEvent):
    event = "merchant_order"
    order_id = StringField("OrderId")
    order_status = IntegerField("OrderStatus")
    product_id = StringField("ProductId")
    sku_info = StringField("SkuInfo")


class KfCreateSessionEvent(BaseEvent):
    event = "kf_create_session"
    account = StringField("KfAccount")


class KfCloseSessionEvent(BaseEvent):
    event = "kf_close_session"
    account = StringField("KfAccount")


class KfSwitchSessionEvent(BaseEvent):
    event = "kf_switch_session"
    from_account = StringField("FromKfAccount")
    to_account = StringField("ToKfAccount")


class DeviceTextEvent(BaseEvent):
    event = "device_text"
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    session_id = StringField("SessionID")
    content = Base64DecodeField("Content")
    open_id = StringField("OpenID")


class DeviceBindEvent(BaseEvent):
    event = "device_bind"
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    session_id = StringField("SessionID")
    content = Base64DecodeField("Content")
    open_id = StringField("OpenID")


class DeviceUnbindEvent(BaseEvent):
    event = "device_unbind"
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    session_id = StringField("SessionID")
    content = Base64DecodeField("Content")
    open_id = StringField("OpenID")


class DeviceSubscribeStatusEvent(BaseEvent):
    event = "device_subscribe_status"
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    open_id = StringField("OpenID")
    op_type = IntegerField("OpType")


class DeviceUnsubscribeStatusEvent(BaseEvent):
    event = "device_unsubscribe_status"
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    open_id = StringField("OpenID")
    op_type = IntegerField("OpType")


class ShakearoundUserShakeEvent(BaseEvent):
    event = "shakearoundusershake"
    _chosen_beacon = BaseField("ChosenBeacon", {})
    _around_beacons = BaseField("AroundBeacons", {})

    @property
    def chosen_beacon(self):
        beacon = self._chosen_beacon
        if not beacon:
            return {}
        return {
            "uuid": beacon["Uuid"],
            "major": beacon["Major"],
            "minor": beacon["Minor"],
            "distance": float(beacon["Distance"]),
        }

    @property
    def around_beacons(self):
        beacons = self._around_beacons
        if not beacons:
            return []

        ret = []
        for beacon in beacons["AroundBeacon"]:
            ret.append(
                {
                    "uuid": beacon["Uuid"],
                    "major": beacon["Major"],
                    "minor": beacon["Minor"],
                    "distance": float(beacon["Distance"]),
                }
            )
        return ret


class PoiCheckNotifyEvent(BaseEvent):
    event = "poi_check_notify"
    poi_id = StringField("PoiId")
    uniq_id = StringField("UniqId")
    result = StringField("Result")
    message = StringField("Msg")


class WiFiConnectedEvent(BaseEvent):
    event = "wificonnected"
    connect_time = IntegerField("ConnectTime")
    expire_time = IntegerField("ExpireTime")
    vendor_id = StringField("VendorId")
    shop_id = StringField("PlaceId")
    bssid = StringField("DeviceNo")


# ============================================================================
# 微信认证事件推送
# ============================================================================
class QualificationVerifySuccessEvent(BaseEvent):
    """
    资质认证成功事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Wechat_Accreditation_Event_Push.html
    """

    event = "qualification_verify_success"
    expired_time = DateTimeField("ExpiredTime")


class QualificationVerifyFailEvent(BaseEvent):
    """
    资质认证失败事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Wechat_Accreditation_Event_Push.html
    """

    event = "qualification_verify_fail"
    fail_time = DateTimeField("FailTime")
    fail_reason = StringField("FailReason")


class NamingVerifySuccessEvent(BaseEvent):
    """
    名称认证成功事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Wechat_Accreditation_Event_Push.html
    """

    event = "naming_verify_success"
    expired_time = DateTimeField("ExpiredTime")


class NamingVerifyFailEvent(BaseEvent):
    """
    名称认证失败事件

    客户端不打勾，但仍有接口权限。详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Wechat_Accreditation_Event_Push.html
    """

    event = "naming_verify_fail"
    fail_time = DateTimeField("FailTime")
    fail_reason = StringField("FailReason")


class AnnualRenewEvent(BaseEvent):
    """
    年审通知事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Wechat_Accreditation_Event_Push.html
    """

    event = "annual_renew"
    expired_time = DateTimeField("ExpiredTime")


class VerifyExpiredEvent(BaseEvent):
    """
    认证过期失效通知

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Account_Management/Wechat_Accreditation_Event_Push.html
    """

    event = "verify_expired"
    expired_time = DateTimeField("ExpiredTime")


class UserScanProductEvent(BaseEvent):
    """
    打开商品主页事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """

    event = "user_scan_product"
    standard = StringField("KeyStandard")
    key = StringField("KeyStr")
    country = StringField("Country")
    province = StringField("Province")
    city = StringField("City")
    sex = IntegerField("Sex")
    scene = IntegerField("Scene")


class UserScanProductEnterSessionEvent(BaseEvent):
    """
    进入公众号事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """

    event = "user_scan_product_enter_session"
    standard = StringField("KeyStandard")
    key = StringField("KeyStr")


class UserScanProductAsyncEvent(BaseEvent):
    """
    地理位置信息异步推送事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """

    event = "user_scan_product_async"
    standard = StringField("KeyStandard")
    key = StringField("KeyStr")
    region_code = StringField("RegionCode")


class UserScanProductVerifyActionEvent(BaseEvent):
    """
    商品审核结果事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """

    event = "user_scan_product_verify_action"
    standard = StringField("KeyStandard")
    key = StringField("KeyStr")
    result = StringField("Result")
    reason = StringField("ReasonMsg")


class SubscribeScanProductEvent(BaseEvent):
    """
    用户在商品主页中关注公众号事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """

    event = "subscribe_scan_product"
    event_key = StringField("EventKey")

    @property
    def scene(self):
        return self.event_key.split("|", 1)[0]

    @property
    def standard(self):
        return self.event_key.split("|")[1]

    @property
    def key(self):
        return self.event_key.split("|")[2]


class UserAuthorizeInvoiceEvent(BaseEvent):
    """
    用户授权发票事件
    （会包含一个订单号，不成功就失败）

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2
    """

    event = "user_authorize_invoice"
    success_order_id = StringField("SuccOrderId")  # 授权成功的订单号
    fail_order_id = StringField("FailOrderId")  # 授权失败的订单号
    app_id = StringField("AppId")  # 用于接收事件推送的公众号的AppId
    auth_source = StringField("Source")  # 授权来源，web表示来自微信内H5，app标识来自app


class UpdateInvoiceStatusEvent(BaseEvent):
    """
    发票状态更新事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2
    """

    event = "update_invoice_status"
    status = StringField("Status")  # 发票报销状态
    card_id = StringField("CardId")  # 发票 Card ID
    code = StringField("Code")  # 发票 Code


class SubmitInvoiceTitleEvent(BaseEvent):
    """
    用户提交发票抬头事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1496554912_vfWU0
    """

    event = "submit_invoice_title"
    title = StringField("title")  # 抬头
    phone = StringField("phone")  # 联系方式
    tax_no = StringField("tax_no")  # 税号
    addr = StringField("addr")  # 地址
    bank_type = StringField("bank_type")  # 银行类型
    bank_no = StringField("bank_no")  # 银行号码
    attach = StringField("attach")  # 附加字段
    title_type = StringField("title_type")  # 抬头类型，个人InvoiceUserTitlePersonType, 公司InvoiceUserTitleBusinessType


class UserEnterTempSessionEvent(BaseEvent):
    """
    小程序用户进入客服消息
    详情请参阅
    https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/customer-message/receive.html
    """

    event = "user_enter_tempsession"
    session_from = StringField("SessionFrom")


class ViewMiniProgramEvent(BaseEvent):
    """
    从菜单进入小程序事件
    """

    event = "view_miniprogram"
    page_path = StringField("EventKey")  # 小程序路径
    menu_id = StringField("MenuId")  # 菜单ID


class WxaMediaCheckEvent(BaseEvent):
    """
    异步检测结果通知事件
    详情请参考
    https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/sec-check/security.mediaCheckAsync.html
    """

    event = "wxa_media_check"
    is_risky = IntegerField("isrisky")  # 检测结果，0：暂未检测到风险，1：风险
    extra_info_json = StringField("extra_info_json")  # 附加信息，默认为空
    trace_id = StringField("trace_id")  # 任务 id
    status_code = IntegerField("status_code")  # 默认为：0，4294966288(-1008)为链接无法下载

    @property
    def is_valid(self):
        return self.is_risky == 0 and self.status_code == 0



# -*- coding: utf-8 -*-


import xmltodict

from wechatpy.work.events import EVENT_TYPES
from wechatpy.work.messages import MESSAGE_TYPES
from wechatpy.messages import UnknownMessage
from wechatpy.utils import to_text


def parse_message(xml):
    if not xml:
        return
    message = xmltodict.parse(to_text(xml))["xml"]
    message_type = message["MsgType"].lower()
    if message_type == "event":
        event_type = message["Event"].lower()
        message_class = EVENT_TYPES.get(event_type, UnknownMessage)
    else:
        message_class = MESSAGE_TYPES.get(message_type, UnknownMessage)
    return message_class(message)



# -*- coding: utf-8 -*-


from wechatpy.exceptions import WeChatClientException
from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMenu(BaseWeChatAPI):
    def get(self):
        """
        查询自定义菜单。

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Querying_Custom_Menus.html

        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            menu = client.menu.get()

        """
        try:
            return self._get("menu/get")
        except WeChatClientException as e:
            if e.errcode == 46003:
                # menu not exist
                return None
            else:
                raise e

    def create(self, menu_data):
        """
        创建自定义菜单 ::

            from wechatpy import WeChatClient

            client = WeChatClient("appid", "secret")
            client.menu.create({
                "button":[
                    {
                        "type":"click",
                        "name":"今日歌曲",
                        "key":"V1001_TODAY_MUSIC"
                    },
                    {
                        "type":"click",
                        "name":"歌手简介",
                        "key":"V1001_TODAY_SINGER"
                    },
                    {
                        "name":"菜单",
                        "sub_button":[
                            {
                                "type":"view",
                                "name":"搜索",
                                "url":"http://www.soso.com/"
                            },
                            {
                                "type":"view",
                                "name":"视频",
                                "url":"http://v.qq.com/"
                            },
                            {
                                "type":"click",
                                "name":"赞一下我们",
                                "key":"V1001_GOOD"
                            }
                        ]
                    }
                ]
            })

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Creating_Custom-Defined_Menu.html

        :param menu_data: Python 字典

        :return: 返回的 JSON 数据包
        """
        return self._post("menu/create", data=menu_data)

    def update(self, menu_data):
        """
        更新自定义菜单 ::

            from wechatpy import WeChatClient

            client = WeChatClient("appid", "secret")
            client.menu.update({
                "button":[
                    {
                        "type":"click",
                        "name":"今日歌曲",
                        "key":"V1001_TODAY_MUSIC"
                    },
                    {
                        "type":"click",
                        "name":"歌手简介",
                        "key":"V1001_TODAY_SINGER"
                    },
                    {
                        "name":"菜单",
                        "sub_button":[
                            {
                                "type":"view",
                                "name":"搜索",
                                "url":"http://www.soso.com/"
                            },
                            {
                                "type":"view",
                                "name":"视频",
                                "url":"http://v.qq.com/"
                            },
                            {
                                "type":"click",
                                "name":"赞一下我们",
                                "key":"V1001_GOOD"
                            }
                        ]
                    }
                ]
            })

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Creating_Custom-Defined_Menu.html

        :param menu_data: Python 字典

        :return: 返回的 JSON 数据包
        """
        return self.create(menu_data)

    def delete(self):
        """
        删除自定义菜单。

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Deleting_Custom-Defined_Menu.html

        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.menu.delete()

        """
        return self._get("menu/delete")

    def get_menu_info(self):
        """
        获取自定义菜单配置

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Getting_Custom_Menu_Configurations.html

        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            menu_info = client.menu.get_menu_info()

        """
        return self._get("get_current_selfmenu_info")

    def add_conditional(self, menu_data):
        """
        创建个性化菜单 ::

            from wechatpy import WeChatClient

            client = WeChatClient("appid", "secret")
            client.menu.add_conditional({
                "button":[
                    {
                        "type":"click",
                        "name":"今日歌曲",
                        "key":"V1001_TODAY_MUSIC"
                    },
                    {
                        "type":"click",
                        "name":"歌手简介",
                        "key":"V1001_TODAY_SINGER"
                    },
                    {
                        "name":"菜单",
                        "sub_button":[
                            {
                                "type":"view",
                                "name":"搜索",
                                "url":"http://www.soso.com/"
                            },
                            {
                                "type":"view",
                                "name":"视频",
                                "url":"http://v.qq.com/"
                            },
                            {
                                "type":"click",
                                "name":"赞一下我们",
                                "key":"V1001_GOOD"
                            }
                        ]
                    }
                ],
                "matchrule":{
                  "group_id":"2",
                  "sex":"1",
                  "country":"中国",
                  "province":"广东",
                  "city":"广州",
                  "client_platform_type":"2"
                }
            })

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Personalized_menu_interface.html

        :param menu_data: Python 字典

        :return: 返回的 JSON 数据包
        """
        return self._post("menu/addconditional", data=menu_data)

    def del_conditional(self, menu_id):
        """
        删除个性化菜单

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Personalized_menu_interface.html

        :param menu_id: 菜单ID

        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.menu.del_conditional('menu_id')

        """
        return self._post("menu/delconditional", data={"menuid": menu_id})

    def try_match(self, user_id):
        """
        测试个性化菜单匹配结果

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Custom_Menus/Personalized_menu_interface.html

        :param user_id: 可以是粉丝的OpenID，也可以是粉丝的微信号。

        :return: 该接口将返回菜单配置

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.menu.try_match('openid')

        """
        return self._post("menu/trymatch", data={"user_id": user_id})



# -*- coding: utf-8 -*-

from typing import Optional

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatUser(BaseWeChatAPI):
    """
    成员管理

    https://developer.work.weixin.qq.com/document/path/90195

    邀请成员接口位于 `WeChatBatch.invite`
    """

    def create(
        self,
        user_id,
        name,
        department=None,
        position=None,
        mobile=None,
        gender=0,
        tel=None,
        email=None,
        weixin_id=None,
        extattr=None,
        **kwargs,
    ):
        """
        创建成员

        https://work.weixin.qq.com/api/doc#90000/90135/90195
        """
        user_data = optionaldict()
        user_data["userid"] = user_id
        user_data["name"] = name
        user_data["gender"] = gender
        user_data["department"] = department
        user_data["position"] = position
        user_data["mobile"] = mobile
        user_data["tel"] = tel
        user_data["email"] = email
        user_data["weixinid"] = weixin_id
        user_data["extattr"] = extattr
        user_data.update(kwargs)

        return self._post("user/create", data=user_data)

    def get(self, user_id):
        """
        读取成员

        https://developer.work.weixin.qq.com/document/path/90196
        """
        return self._get("user/get", params={"userid": user_id})

    def update(
        self,
        user_id,
        name=None,
        department=None,
        position=None,
        mobile=None,
        gender=None,
        tel=None,
        email=None,
        weixin_id=None,
        enable=None,
        extattr=None,
        **kwargs,
    ):
        """
        更新成员

        https://developer.work.weixin.qq.com/document/path/90197
        """
        user_data = optionaldict()
        user_data["userid"] = user_id
        user_data["name"] = name
        user_data["gender"] = gender
        user_data["department"] = department
        user_data["position"] = position
        user_data["mobile"] = mobile
        user_data["tel"] = tel
        user_data["email"] = email
        user_data["weixinid"] = weixin_id
        user_data["extattr"] = extattr
        user_data["enable"] = enable
        user_data.update(kwargs)

        return self._post("user/update", data=user_data)

    def delete(self, user_id):
        """
        删除成员

        https://developer.work.weixin.qq.com/document/path/90198
        """
        return self._get("user/delete", params={"userid": user_id})

    def batch_delete(self, user_ids):
        """
        批量删除成员

        https://developer.work.weixin.qq.com/document/path/90199
        """
        return self._post("user/batchdelete", data={"useridlist": user_ids})

    def list(self, department_id, fetch_child=False, status=0, simple=False):
        """
        批量获取部门成员 / 批量获取部门成员详情

        https://developer.work.weixin.qq.com/document/path/90200
        https://developer.work.weixin.qq.com/document/path/90201

        此接口和 `WeChatDepartment.get_users` 是同一个接口，区别为 simple 的默认值不同。
        """
        url = "user/simplelist" if simple else "user/list"
        res = self._get(
            url,
            params={
                "department_id": department_id,
                "fetch_child": 1 if fetch_child else 0,
                "status": status,
            },
        )
        return res["userlist"]

    def convert_to_openid(self, user_id, agent_id=None):
        """
        user_id 转成 openid

        https://developer.work.weixin.qq.com/document/path/90202

        :param user_id: 企业微信内的成员 ID
        :param agent_id: 可选，需要发送红包的应用ID，若只是使用微信支付和企业转账，则无需该参数
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(userid=user_id, agentid=agent_id)
        return self._post("user/convert_to_openid", data=data)

    def convert_to_user_id(self, openid):
        """
        openid 转成 user_id

        https://developer.work.weixin.qq.com/document/path/90202

        :param openid: 在使用微信支付、微信红包和企业转账之后，返回结果的openid
        :return: 该 openid 在企业微信中对应的成员 user_id
        """
        res = self._post("user/convert_to_userid", data={"openid": openid})
        return res["userid"]

    def verify(self, user_id):
        """
        二次验证

        https://developer.work.weixin.qq.com/document/path/90203

        :param user_id: 成员UserID。对应管理端的帐号
        """
        return self._get("user/authsucc", params={"userid": user_id})

    def get_join_qrcode(self, size_type: Optional[int] = None) -> str:
        """
        获取加入企业二维码

        该接口用于获取企业用户实时成员加入的二维码。详细接口细节请查看 `接口文档`_。

        需要注意的是获取到的二维码链接，**有效期为7天**。

        :param size_type: 图片尺寸类型。可用尺寸有：
            1: 171 x 171;
            2: 399 x 399;
            3: 741 x 741（默认）;
            4: 2052 x 2052
        :return: 二维码链接

        .. _接口文档: https://developer.work.weixin.qq.com/document/path/91714

        .. warning:: 使用本接口请确保开启了 **通讯录同步** 的API接口同步，并使用
          **通讯录同步** 的 ``secret``，否则调用接口时会出现错误。
        """
        params = optionaldict(size_type=size_type)
        resp = self._get("corp/get_join_qrcode", params=params)
        return resp["join_qrcode"]

    def get_active_stat(self, date: str) -> int:
        """
        获取企业活跃成员数

        该接口用于获取指定日期的企业的成员活跃数量，详细接口细节请查看 `接口文档`_。

        :param date: 具体某天的活跃人数，最长支持获取30天前数据。格式为: ``YYYY-MM-DD``。
        :return: 成员活跃数量

        .. _接口文档:: https://developer.work.weixin.qq.com/document/path/92714

        .. warning:: 仅通讯录同步助手可调用。
        """
        resp = self._post("user/get_active_stat", data={"date": date})
        return resp["active_cnt"]



# -*- coding: utf-8 -*-


from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatAgent(BaseWeChatAPI):
    """应用管理
    https://work.weixin.qq.com/api/doc#90000/90135/90226
    """

    def get(self, agent_id):
        """
        获取指定的应用详情
        https://work.weixin.qq.com/api/doc#90000/90135/90227/获取指定的应用详情/

        :param agent_id: 应用id
        :return: 返回的 JSON 数据包
        """
        return self._get("agent/get", params={"agentid": agent_id})

    def list(self):
        """
        获取access_token对应的应用列表
        https://work.weixin.qq.com/api/doc#90000/90135/90227/获取access_token对应的应用列表/

        :return: 应用概况列表
        """
        res = self._get("agent/list")
        return res["agentlist"]

    def set(
        self,
        agent_id,
        name=None,
        description=None,
        redirect_domain=None,
        logo_media_id=None,
        report_location_flag=0,
        is_report_user=True,
        is_report_enter=True,
    ):
        """
        设置应用
        https://work.weixin.qq.com/api/doc#90000/90135/90228

        :param agent_id: 企业应用的id
        :param name: 企业应用名称，长度不超过32个utf8字符
        :param description: 企业应用详情，长度为4至120个utf8字符
        :param redirect_domain: 企业应用可信域名。注意：域名需通过所有权校验，否则jssdk功能将受限，此时返回错误码85005
        :param logo_media_id: 企业应用头像的mediaid，通过素材管理接口上传图片获得mediaid，上传后会自动裁剪成方形和圆形两个头像
        :param report_location_flag: 企业应用是否打开地理位置上报 0：不上报；1：进入会话上报；
        :param is_report_enter: 是否上报用户进入应用事件。0：不接收；1：接收。
        :param is_report_user: 是否接收用户变更通知。0：不接收；1：接收。
        :return: 返回的 JSON 数据包
        """
        agent_data = optionaldict()
        agent_data["agentid"] = agent_id
        agent_data["name"] = name
        agent_data["description"] = description
        agent_data["redirect_domain"] = redirect_domain
        agent_data["logo_mediaid"] = logo_media_id
        agent_data["report_location_flag"] = report_location_flag
        agent_data["isreportenter"] = 1 if is_report_enter else 0
        agent_data["isreportuser"] = 1 if is_report_user else 0
        return self._post("agent/set", data=agent_data)



# -*- coding: utf-8 -*-
from typing import Optional, List, Iterator

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatExternalContact(BaseWeChatAPI):
    """
    外部联系人管理

    详细说明请查阅企业微信有关 `外部联系人管理`_ 的文档。

    .. _外部联系人管理: https://work.weixin.qq.com/api/doc/90000/90135/92109

    .. _客户联系secret: https://work.weixin.qq.com/api/doc/90000/90135/92570
            #13473/%E5%BC%80%E5%A7%8B%E5%BC%80%E5%8F%91

    .. _可调用应用: https://work.weixin.qq.com/api/doc/90000/90135/92570#134
        73/%E5%BC%80%E5%A7%8B%E5%BC%80%E5%8F%91

    .. _客户联系功能: https://work.weixin.qq.com/api/doc/90000/90135/92125
        #13473/%E5%BC%80%E5%A7%8B%E5%BC%80%E5%8F%91

    .. _企业客户权限: https://work.weixin.qq.com/api/doc/90000/90135/92572#19519

    .. _获取外部联系人详情: https://work.weixin.qq.com/api/doc/90000/90135/92572
        #13878
    """

    def get_follow_user_list(self) -> dict:
        """
        获取配置了客户联系功能的成员列表

        企业和第三方服务商可获取配置了客户联系功能的成员列表。

        详细请查阅企业微信官方文档 `获取配置了客户联系功能的成员列表`_ 章节。

        使用示例:

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 获取成员用户userid列表数据
            follow_users = client.external_contact.get_follow_user_list()["follow_user"]

        :return: 配置了客户联系功能的成员用户userid信息

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需拥有“企业客户”权限。
            - 第三方/自建应用只能获取到可见范围内的配置了客户联系功能的成员。

        .. _获取配置了客户联系功能的成员列表:
            https://work.weixin.qq.com/api/doc/90000/90135/92570
        """
        return self._get("externalcontact/get_follow_user_list")

    def list(self, userid: str) -> dict:
        """
        获取客户列表

        企业可通过此接口获取指定成员添加的客户列表。客户是指配置了客户联系功能的成员所添加的
        外部联系人。没有配置客户联系功能的成员，所添加的外部联系人将不会作为客户返回。

        详细请查阅企业微信官方文档 `获取客户列表`_ 章节。

        使用示例:

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 获取外部联系人的userid列表
            follow_users = client.external_contact.list("user_id")["external_userid"]

        :param userid: 企业成员的userid
        :return: 包含外部联系人的userid列表的字典类型数据

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需拥有“企业客户”权限。
            - 第三方/自建应用只能获取到可见范围内的配置了客户联系功能的成员。

        .. _获取客户列表: https://work.weixin.qq.com/api/doc/90000/90135/92113
        """
        return self._get("externalcontact/list", params={"userid": userid})

    def batch_get_by_user(self, userid: str, cursor: str = "", limit: int = 50) -> dict:
        """
        批量获取客户详情

        使用示例：

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 批量获取该企业员工添加的客户(外部联系人)的详情
            external_contact_list = client.external_contact.batch_get_by_user("user_id", "cursor", 10)["external_contact_list"]

        :param userid: 企业成员的userid
        :param cursor: 用于分页查询的游标，字符串类型，由上一次调用返回，首次调用可不填
        :param limit: 返回的最大记录数，整型，最大值100，默认值50，超过最大值时取最大值
        :return: 包含该企业员工添加的部分客户详情列表的字典类型数据

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户基础信息”权限
            - 第三方/自建应用调用此接口时，userid需要在相关应用的可见范围内。

        .. _批量获取客户详情: https://work.weixin.qq.com/api/doc/90000/90135/92994
        """
        data = optionaldict(
            userid=userid,
            cursor=cursor,
            limit=limit,
        )
        return self._post("externalcontact/batch/get_by_user", data=data)

    def gen_all_by_user(self, userid: str, limit: int = 50) -> Iterator[dict]:
        """
        获取企业员工添加的所有客户详情列表的生成器

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            #  获取企业员工添加的所有客户详情列表
            for i in client.external_contact.gen_all_by_user("user_id", 10):
                print(i)

        :param userid: 企业员工userid
        :param limit: 每次需要请求微信接口时返回的最大记录数，整型，最大值100，默认值50，超过最大值时取最大值
        :return: 企业员工添加的所有客户详情列表的生成器

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户基础信息”权限
            - 第三方/自建应用调用此接口时，userid需要在相关应用的可见范围内。
        """
        cursor = ""
        while True:
            response = self.batch_get_by_user(userid, cursor, limit)
            if response.get("errcode") == 0:
                yield from response.get("external_contact_list", [])
            if response.get("next_cursor"):
                cursor = response["next_cursor"]
            else:
                break

    def get(self, external_userid: str) -> dict:
        """
        获取客户详情

        企业可通过此接口，根据 `外部联系人的userid（如何获取?）`_，拉取客户详情。

        详细请查阅企业微信官方文档 `获取客户详情`_ 章节。

        使用示例:

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 接口数据
            data = client.external_contact.get("external_userid")
            # 外部联系人的自定义展示信息
            external_profile = data["external_profile"]  # type: dict
            # 部联系人的企业成员userid
            follow_users = data["follow_user"]  # type: List[dict]

        :param external_userid: 外部联系人的userid，注意不是企业成员的帐号
        :return: 用户信息（字段内容请参考官方文档 `获取客户详情`_ 章节）

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方/自建应用调用时，返回的跟进人follow_user仅包含应用可见范围之内的成员。

        .. _外部联系人的userid（如何获取?）: https://work.weixin.qq.com/api/doc/9
            0000/90135/92114#15445

        .. _获取客户详情: https://work.weixin.qq.com/api/doc/90000/90135/92114
        """
        return self._get("externalcontact/get", params={"external_userid": external_userid})

    def add_contact_way(
        self,
        type: int,
        scene: int,
        style: Optional[int] = None,
        remark: Optional[str] = None,
        skip_verify: bool = True,
        state: Optional[str] = None,
        user: List[str] = None,
        party: List[int] = None,
        is_temp: bool = False,
        expires_in: Optional[int] = None,
        chat_expires_in: Optional[int] = None,
        unionid: Optional[str] = None,
        conclusions: Optional[dict] = None,
    ) -> dict:
        """
        配置客户联系「联系我」方式

        详细请查阅企业微信官方文档 `配置客户联系「联系我」方式`_ 章节。

        **注意:**

        - 每个联系方式最多配置100个使用成员（包含部门展开后的成员）
        - 当设置为临时会话模式时（即 ``is_temp`` 为 `True` ），联系人仅支持配置为单人，
          暂不支持多人
        - 使用 ``unionid`` 需要调用方（企业或服务商）的企业微信“客户联系”中已绑定微信开
          发者账户

        使用示例:

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 调用接口
            result = client.external_contact.add_contact_way(
                type=1,
                scene=1,
                style=1,
                remark="渠道用户",
                skip_verify=True,
                state="teststate",
                user=["zhansan", "lisi"],
                party=[2,3],
                is_temp=True,
                expires_in=86400,
                chat_expires_in=86400,
                unionid="oxTWIuGaIt6gTKsQRLau2M0AAAA",
                conclusions={
                    "text": {"content": "文本消息内容"},
                    "image": {"media_id": "MEDIA_ID"},
                    "link": {
                        "title": "消息标题",
                        "picurl": "https://example.pic.com/path",
                        "desc": "消息描述",
                        "url": "https://example.link.com/path",
                    },
                    "miniprogram": {
                        "title": "消息标题",
                        "pic_media_id": "MEDIA_ID",
                        "appid": "wx8bd80126147dfAAA",
                        "page": "/path/index.html",
                    },
                },
            )
            # 新增联系方式的配置id
            config_id = result["config_id"]
            # 联系我二维码链接，仅在scene为2时返回
            qr_code = result.get("qr_code")

        :param type: 联系方式类型,1-单人, 2-多人
        :param scene: 场景，1-在小程序中联系，2-通过二维码联系
        :param style: 在小程序中联系时使用的控件样式，详见附表
        :param remark: 联系方式的备注信息，用于助记，不超过30个字符
        :param skip_verify: 外部客户添加时是否无需验证，默认为true
        :param state: 企业自定义的state参数，用于区分不同的添加渠道，在调用
            `获取外部联系人详情`_ 时会返回该参数值
        :param user: 使用该联系方式的用户userID列表，在type为1时为必填，且只能有一个
        :param party: 使用该联系方式的部门id列表，只在type为2时有效
        :param is_temp: 是否临时会话模式，``True`` 表示使用临时会话模式，默认为 ``False``
        :param expires_in: 临时会话二维码有效期，以秒为单位。该参数仅在 ``is_temp`` 为
            ``True`` 时有效，默认7天
        :param chat_expires_in: 临时会话有效期，以秒为单位。该参数仅在 ``is_temp`` 为
            ``True`` 时有效，默认为添加好友后24小时
        :param unionid: 可进行临时会话的客户unionid，该参数仅在 ``is_temp`` 为
            ``True``时有效，如不指定则不进行限制
        :param conclusions: 结束语，会话结束时自动发送给客户，可参考 `结束语定义`_，
            仅在 ``is_temp`` 为 ``True`` 时有效
        :return: 返回的 JSON 数据包

        .. note::
            **调用接口应满足如下的权限要求：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 使用人员需要配置了 `客户联系功能`_。
            - 第三方调用时，应用需具有 `企业客户权限`_。
            - 第三方/自建应用调用时，传入的userid和partyid需要在此应用的可见范围内。
            - 配置的使用成员必须在企业微信激活且已经过实名认证。
            - 临时会话的二维码具有有效期，添加企业成员后仅能在指定有效期内进行会话，
              仅支持医疗行业企业创建。
              临时会话模式可以配置会话结束时自动发送给用户的结束语。

        .. _配置客户联系「联系我」方式: https://work.weixin.qq.com/api/doc/90000/9
            0135/92572#%E9%85%8D%E7%BD%AE%E5%AE%A2%E6%88%B7%E8%81%94%E7%B3%BB
            %E3%80%8C%E8%81%94%E7%B3%BB%E6%88%91%E3%80%8D%E6%96%B9%E5%BC%8F

        .. _结束语定义: https://work.weixin.qq.com/api/doc/90000/90135/92572#156
            45/%E7%BB%93%E6%9D%9F%E8%AF%AD%E5%AE%9A%E4%B9%89
        """
        data = optionaldict(
            type=type,
            scene=scene,
            style=style,
            remark=remark,
            skip_verify=skip_verify,
            state=state,
            user=user,
            party=party,
            is_temp=is_temp,
            expires_in=expires_in,
            chat_expires_in=chat_expires_in,
            unionid=unionid,
            conclusions=conclusions,
        )
        return self._post("externalcontact/add_contact_way", data=data)

    def get_contact_way(self, config_id: str) -> dict:
        """
        获取企业已配置的「联系我」方式

        批量获取企业配置的「联系我」二维码和「联系我」小程序按钮。

        详细请查阅企业微信官方文档 `获取企业已配置的「联系我」方式`_ 章节。

        :param config_id: 联系方式的配置id, e.g.42b34949e138eb6e027c123cba77fad7
        :return: 返回的 JSON 数据包

        .. note::
            **调用接口应满足如下的权限要求：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 使用人员需要配置了 `客户联系功能`_。
            - 第三方调用时，应用需具有 `企业客户权限`_。
            - 第三方/自建应用调用时，传入的userid和partyid需要在此应用的可见范围内。
            - 配置的使用成员必须在企业微信激活且已经过实名认证。
            - 临时会话的二维码具有有效期，添加企业成员后仅能在指定有效期内进行会话，
              仅支持医疗行业企业创建。
              临时会话模式可以配置会话结束时自动发送给用户的结束语。

        .. _获取企业已配置的「联系我」方式: https://work.weixin.qq.com/api/doc/90000
           /90135/92572#%E8%8E%B7%E5%8F%96%E4%BC%81%E4%B8%9A%E5%B7%B2%E9%85%8D
           %E7%BD%AE%E7%9A%84%E3%80%8C%E8%81%94%E7%B3%BB%E6%88%91%E3%80%8D%E6%
           96%B9%E5%BC%8F
        """
        data = optionaldict(config_id=config_id)
        return self._post("externalcontact/get_contact_way", data=data)

    def update_contact_way(
        self,
        config_id,
        remark,
        skip_verify=True,
        style=None,
        state=None,
        user=None,
        party=None,
    ) -> dict:
        """
        更新企业已配置的「联系我」方式

        更新企业配置的「联系我」二维码和「联系我」小程序按钮中的信息，如使用人员和备注等。

        详细请查阅企业微信官方文档 `更新企业已配置的「联系我」方式`_ 章节。

        :param config_id: 企业联系方式的配置id
        :param remark: 联系方式的备注信息，不超过30个字符，将覆盖之前的备注
        :param skip_verify: 外部客户添加时是否无需验证
        :param style: 样式，只针对“在小程序中联系”的配置生效
        :param state: 企业自定义的state参数，用于区分不同的添加渠道，在调用“获取外部联系
           人详情”时会返回该参数值
        :param user: 使用该联系方式的用户列表，将覆盖原有用户列表
        :param party: 使用该联系方式的部门列表，将覆盖原有部门列表，只在配置的type为2时有效
        :return: 返回的 JSON 数据包

        .. note::
            **调用接口应满足如下的权限要求：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 使用人员需要配置了 `客户联系功能`_。
            - 第三方调用时，应用需具有 `企业客户权限`_。
            - 第三方/自建应用调用时，传入的userid和partyid需要在此应用的可见范围内。
            - 配置的使用成员必须在企业微信激活且已经过实名认证。
            - 临时会话的二维码具有有效期，添加企业成员后仅能在指定有效期内进行会话，
              仅支持医疗行业企业创建。
              临时会话模式可以配置会话结束时自动发送给用户的结束语。

        .. _更新企业已配置的「联系我」方式: https://work.weixin.qq.com/api/doc/90000
           /90135/92572#%E6%9B%B4%E6%96%B0%E4%BC%81%E4%B8%9A%E5%B7%B2%E9%85%8D
           %E7%BD%AE%E7%9A%84%E3%80%8C%E8%81%94%E7%B3%BB%E6%88%91%E3%80%8D%E6%
           96%B9%E5%BC%8F
        """
        data = optionaldict(
            config_id=config_id,
            remark=remark,
            skip_verify=skip_verify,
            style=style,
            state=state,
            user=user,
            party=party,
        )
        return self._post("externalcontact/update_contact_way", data=data)

    def del_contact_way(self, config_id: str) -> dict:
        """
        删除企业已配置的「联系我」方式

        删除一个已配置的「联系我」二维码或者「联系我」小程序按钮。

        详细请查阅企业微信官方文档 `删除企业已配置的「联系我」方式`_ 章节。

        :param config_id: 企业联系方式的配置id
        :return: 返回的 JSON 数据包

        .. note::
            **调用接口应满足如下的权限要求：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 使用人员需要配置了 `客户联系功能`_。
            - 第三方调用时，应用需具有 `企业客户权限`_。
            - 第三方/自建应用调用时，传入的userid和partyid需要在此应用的可见范围内。
            - 配置的使用成员必须在企业微信激活且已经过实名认证。
            - 临时会话的二维码具有有效期，添加企业成员后仅能在指定有效期内进行会话，
              仅支持医疗行业企业创建。
              临时会话模式可以配置会话结束时自动发送给用户的结束语。

        .. _删除企业已配置的「联系我」方式: https://work.weixin.qq.com/api/doc/90000
           /90135/92572#%E5%88%A0%E9%99%A4%E4%BC%81%E4%B8%9A%E5%B7%B2%E9%85%8D
           %E7%BD%AE%E7%9A%84%E3%80%8C%E8%81%94%E7%B3%BB%E6%88%91%E3%80%8D%E6%
           96%B9%E5%BC%8F

        """
        data = optionaldict(config_id=config_id)
        return self._post("externalcontact/del_contact_way", data=data)

    def add_msg_template(self, template: dict) -> dict:
        """
        添加企业群发消息模板

        企业可通过此接口添加企业群发消息的任务并通知客服人员发送给相关客户或客户群。
        （注：企业微信终端需升级到2.7.5版本及以上）

        **注意**：调用该接口并不会直接发送消息给客户/客户群，需要相关的客服人员操作以后才会
        实际发送（客服人员的企业微信需要升级到2.7.5及以上版本）

        同一个企业每个自然月内仅可针对一个客户/客户群发送4条消息，超过限制的用户将会被忽略。

        详细请查阅企业微信官方文档 `添加企业群发消息任务`_ 章节。

        使用示例:

        .. code-block:: python

            from wechatpy.exceptions import WeChatClientException
            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            template = {
                "chat_type":"single",
                "external_userid":[
                    "woAJ2GCAAAXtWyujaWJHDDGi0mACAAAA",
                    "wmqfasd1e1927831123109rBAAAA"
                ],
                "sender":"zhangsan",
                "text":{
                    "content":"文本消息内容"
                },
                "image":{
                    "media_id":"MEDIA_ID",
                    "pic_url":"http://p.qpic.cn/pic_wework/3474110808/7a6344sdadfwehe42060/0"
                },
                "link":{
                    "title":"消息标题",
                    "picurl":"https://example.pic.com/path",
                    "desc":"消息描述",
                    "url":"https://example.link.com/path"
                },
                "miniprogram":{
                    "title":"消息标题",
                    "pic_media_id":"MEDIA_ID",
                    "appid":"wx8bd80126147dfAAA",
                    "page":"/path/index.html"
                }
            }
            try:
                result = client.external_contact.add_msg_template(template=template)
                # 无效或无法发送的external_userid列表
                fail_list = result["fail_list"]
                # 企业群发消息的id，可用于获取群发消息发送结果
                msgid = result["msgid]
            except WeChatClientException as err:
                # 接口调用失败
                ...

        :param template: 参考官方文档和使用示例
        :return: 请求结果（字典类型）

        .. _添加企业群发消息任务: https://work.weixin.qq.com/api/doc/90000/90135/92135
        """
        return self._post("externalcontact/add_msg_template", data=template)

    def get_group_msg_result(self, msgid):
        """
        获取企业群发消息发送结果

        企业和第三方可通过该接口获取到添加企业群发消息模板生成消息的群发发送结果。
        https://work.weixin.qq.com/api/doc#90000/90135/91561

        :param msgid: 群发消息的id，通过添加企业群发消息模板接口返回
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(msgid=msgid)
        return self._post("externalcontact/get_group_msg_result", data=data)

    def get_user_behavior_data(
        self,
        userid: Optional[List[str]],
        start_time: int,
        end_time: int,
        partyid: Optional[List[str]] = None,
    ) -> dict:
        """
        获取「联系客户统计」数据

        企业可通过此接口获取成员联系客户的数据，包括发起申请数、新增客户数、聊天数、发送消息
        数和删除/拉黑成员的客户数等指标。

        详细请查阅企业微信官方文档 `获取「联系客户统计」数据`_ 章节。

        :param userid: userid列表
        :param partyid: 部门ID列表，最多100个
        :param start_time: 数据起始时间
        :param end_time: 数据结束时间
        :return: 返回的 JSON 数据包
        :raises AssertionError: 当userid和partyid同时为空时抛出该移除

        .. warning::

           1. ``userid`` 和 ``partyid`` 不可同时为空;
           2. 此接口提供的数据以天为维度，查询的时间范围为 ``[START_TIME,END_TIME]``，
              即前后均为闭区间，支持的最大查询跨度为30天；
           3. 用户最多可获取最近180天内的数据；
           4. 当传入的时间不为0点时间戳时，会向下取整，如传入
              1554296400(wED aPR 3 21:00:00 cst 2019) 会被自动转换为
              1554220800（wED aPR 3 00:00:00 cst 2019）;
           5. 如传入多个 ``USERID``，则表示获取这些成员总体的联系客户数据。

        .. note::

            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用使用，需具有“企业客户权限->客户联系->获取成员联系客户的数据统计”权限。
            - 第三方/自建应用调用时传入的userid和partyid要在应用的可见范围内;

        .. _获取「联系客户统计」数据: https://work.weixin.qq.com/api/doc/90000/90135/92132
        """
        assert userid or partyid, "userid和partyid不可同时为空"

        data = optionaldict(userid=userid, start_time=start_time, end_time=end_time, partyid=partyid)
        return self._post("externalcontact/get_user_behavior_data", data=data)

    def send_welcome_msg(self, template: dict) -> dict:
        """
        发送新客户欢迎语

        企业微信在向企业推送 `添加外部联系人事件`_ 时，会额外返回一个welcome_code，企业以
        此为凭据调用接口，即可通过成员向新添加的客户发送个性化的欢迎语。

        为了保证用户体验以及避免滥用，企业仅可在收到相关事件后20秒内调用，且只可调用一次。如
        果企业已经在管理端为相关成员配置了可用的欢迎语，则推送添加外部联系人事件时不会返回
        welcome_code。

        每次添加新客户时 **可能有多个企业自建应用/第三方应用收到带有welcome_code的回调事件**，
        但仅有最先调用的可以发送成功。后续调用将返回 **41051（externaluser has started
        chatting）** 错误，请用户根据实际使用需求，合理设置应用可见范围，避免冲突。

        详细请查阅企业微信官方文档 `发送新客户欢迎语`_ 章节。

        使用示例:

        .. code-block:: python

            from wechatpy.exceptions import WeChatClientException
            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            template = {
                "chat_type":"single",
                "external_userid":[
                    "woAJ2GCAAAXtWyujaWJHDDGi0mACAAAA",
                    "wmqfasd1e1927831123109rBAAAA"
                ],
                "sender":"zhangsan",
                "text":{
                    "content":"文本消息内容"
                },
                "image":{
                    "media_id":"MEDIA_ID",
                    "pic_url":"http://p.qpic.cn/pic_wework/3474110808/7a6344sdadfwehe42060/0"
                },
                "link":{
                    "title":"消息标题",
                    "picurl":"https://example.pic.com/path",
                    "desc":"消息描述",
                    "url":"https://example.link.com/path"
                },
                "miniprogram":{
                    "title":"消息标题",
                    "pic_media_id":"MEDIA_ID",
                    "appid":"wx8bd80126147dfAAA",
                    "page":"/path/index.html"
                }
            }
            try:
                client.external_contact.send_welcome_msg(template=template)
            except WeChatClientException as err:
                # 消息发送失败时的处理
                ...

        :param template: 参考官方文档和使用示例
        :return: 消息推送结果（字典类型）

        .. _添加外部联系人事件: https://work.weixin.qq.com/api/doc/90000/90135/921
            37#15260/%E6%B7%BB%E5%8A%A0%E5%A4%96%E9%83%A8%E8%81%94%E7%B3%BB%E4%
            BA%BA%E4%BA%8B%E4%BB%B6

        .. _发送新客户欢迎语: https://work.weixin.qq.com/api/doc/90000/90135/92137

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需要拥有「企业客户」权限，且企业成员处于相关应用的可见范围内。
        """
        return self._post("externalcontact/send_welcome_msg", data=template)

    def get_unassigned_list(self, page_id: int = 0, page_size: int = 1000, cursor: Optional[str] = None) -> dict:
        """
        获取离职成员列表

        企业和第三方可通过此接口，获取所有离职成员的客户列表，并可进一步调用
        `分配在职或离职成员的客户`_ 接口将这些客户重新分配给其他企业成员。

        详细请查阅企业微信官方文档 `获取离职成员列表`_ 章节。

        :param page_id: 分页查询，要查询页号，从0开始
        :param page_size: 每次返回的最大记录数，默认为1000，最大值为1000
        :param cursor: 分页查询游标，字符串类型，适用于数据量较大的情况，如果使用该参数
           则无需填写page_id，该参数由上一次调用返回
        :return: 响应结果

        .. note::
           当 ``page_id`` 为1，``page_size`` 为100时，表示取第101到第200条记录。
           由于每个成员的客户数不超过5万，故 ``page_id * page_size`` 必须小于5万。

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需拥有“企业客户权限->客户联系->分配在职或离职成员的客户”权限

        .. _获取离职成员列表: https://work.weixin.qq.com/api/doc/90000/90135/92124
        """
        data = optionaldict(page_id=page_id, page_size=page_size, cursor=cursor)
        return self._post("externalcontact/get_unassigned_list", data=data)

    def transfer(
        self,
        external_userid: str,
        handover_userid: str,
        takeover_userid: str,
        transfer_success_msg: Optional[str] = None,
    ) -> dict:
        """
        分配在职或离职成员的客户


        企业可通过此接口，转接在职成员的客户或分配离职成员的客户给其他成员。

        详细请查阅企业微信官方文档 `分配在职或离职成员的客户`_ 章节。

        **调用参数注意事项：**

        - 在某些特殊情景下，可能存在已离职的成员和当前在职的企业成员具有相同userid的情况，
          此时优先分配在职成员的客户.
        - ``external_userid`` 必须是 ``handover_userid`` 的客户
          （即 `配置了客户联系功能`_ 的成员所添加的联系人）。
        - 在职成员的每位客户最多被分配2次。客户被转接成功后，将有90个自然日的服务关系保护期，
          保护期内的客户无法再次被分配。

        使用示例:

        .. code-block:: python

            from wechatpy.exceptions import WeChatClientException
            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            try:
                client.external_contact.transfer(
                    external_userid="woAJ2GCAAAXtWyujaWJHDDGi0mACAAAA",
                    handover_userid="zhangsan",
                    takeover_userid="lisi",
                    transfer_success_msg="您好！",您好
                )
            except WeChatClientException as err:
                # 分配失败时的处理
                ...

        :param external_userid: 外部联系人的userid，注意不是企业成员的帐号
        :param handover_userid: 离职成员的userid
        :param takeover_userid: 接替成员的userid
        :param transfer_success_msg: 转移成功后发给客户的消息，最多200个字符，不填则
            使用默认文案，目前只对在职成员分配客户的情况生效
        :return: 分配结果（字典类型）

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需拥有 `企业客户权限`_。
            - 接替成员必须在此第三方应用或自建应用的可见范围内。
            - 接替成员需要配置了 `客户联系功能`_。
            - 接替成员需要在企业微信激活且已经过实名认证。

        .. _分配在职或离职成员的客户: https://work.weixin.qq.com/api/doc/90000/90135/92125

        .. _配置了客户联系功能: https://work.weixin.qq.com/api/doc/90000/90135/
            92125#13473/%E5%BC%80%E5%A7%8B%E5%BC%80%E5%8F%91


        """
        data = optionaldict(
            external_userid=external_userid,
            handover_userid=handover_userid,
            takeover_userid=takeover_userid,
            transfer_success_msg=transfer_success_msg,
        )
        return self._post("externalcontact/transfer", data=data)

    def get_corp_tag_list(self, tag_ids: Optional[List[str]] = None) -> dict:
        """
        获取企业标签库

        企业可通过此接口获取企业客户标签详情。

        **企业客户标签** 是针对企业的外部联系人进行标记和分类的标签，由企业统一配置后，企业
        成员可使用此标签对客户进行标记。

        详细请查阅企业微信官方文档 `获取企业标签库`_ 章节。

        使用示例:

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 接口数据
            data = client.external_contact.get_corp_tag_list(["tag_id1", "tag_id2"])
            # 标签组列表
            tag_groups = data["tag_group"]  # type: List[dict]

        :param tag_ids: 需要查询的标签id，如果为 ``None`` 则获取该企业的所有客户标签，
            目前暂不支持标签组id。
        :return: 包含标签信息的字典类型数据（详细字段请参考 `获取企业标签库`_）

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 对于添加/删除/编辑企业客户标签接口，目前仅支持使用 `客户联系secret`_ 所获
              取的accesstoken来调用。
            - 第三方仅可读取，且应用需具有 `企业客户权限`_。

        .. _获取企业标签库: https://work.weixin.qq.com/api/doc/90000/90135/92117
            #%E8%8E%B7%E5%8F%96%E4%BC%81%E4%B8%9A%E6%A0%87%E7%AD%BE%E5%BA%93
        """
        data = optionaldict(tag_id=tag_ids)
        return self._post("externalcontact/get_corp_tag_list", data=data)

    def add_corp_tag(
        self,
        group_id: Optional[str],
        group_name: Optional[str],
        order: Optional[int],
        tags: dict,
    ) -> dict:
        """
        添加企业客户标签

        **企业客户标签** 是针对企业的外部联系人进行标记和分类的标签，由企业统一配置后，企业
        成员可使用此标签对客户进行标记。

        企业可通过此接口向客户标签库中添加新的标签组和标签，**每个企业最多可配置3000个企业标签**。

        详细请查阅企业微信官方文档 `添加企业客户标签`_ 章节。

        **参数注意事项:**

        - 如果要向指定的标签组下添加标签，需要提供 ``group_id`` 参数；如果要创建一个全新的
          标签组以及标签，则需要通过 ``group_name`` 参数指定新标签组名称，如果填写的
          ``group_name`` 已经存在，则会在此标签组下新建标签。
        - 如果提供了 ``group_id`` 参数，则 ``group_name`` 和标签组的 ``order`` 参数
          会被忽略。
        - 不支持创建空标签组。
        - 标签组内的标签不可同名，如果传入多个同名标签，则只会创建一个。

        使用示例:

        .. code-block:: python

            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 创建标签
            result = client.external_contact.add_corp_tag(
                group_id="GROUP_ID",
                group_name="GROUP_NAME",
                order=1,
                tags=[
                    {"name": "TAG_NAME_1", "order": 1},
                    {"name": "TAG_NAME_2", "order": 2},
                ],
            )
            # 创建成功后的标签组信息
            tag_group = result["tag_group"]  # type: dict

        :param group_id: 标签组id
        :param group_name: 标签组名称，最长为30个字符
        :param order: 标签组次序值。order值大的排序靠前。有效的值范围是[0, 2^32)
        :param tags: 需要添加的标签列表，标签信息是包含 ``name`` （必须）和 ``order``
            （可选）两个字段的字典类型数据，比如: ``[{"name": 'tag_name", "order": 1}]``
            ，或者 ``[{"name": "tag_name"}]``。
        :return: 标签创建结果（字典类型，字段请参考 `添加企业客户标签`_）

        .. _添加企业客户标签: https://work.weixin.qq.com/api/doc/90000/90135/921
            17#%E6%B7%BB%E5%8A%A0%E4%BC%81%E4%B8%9A%E5%AE%A2%E6%88%B7%E6%A0%8
            7%E7%AD%BE

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 对于添加/删除/编辑企业客户标签接口，目前仅支持使用 `客户联系secret`_ 所获
              取的accesstoken来调用。
            - 第三方仅可读取，且应用需具有 `企业客户权限`_。

        .. warning:: 暂不支持第三方调用。
        """
        data = optionaldict(group_id=group_id, group_name=group_name, order=order, tag=tags)

        return self._post("externalcontact/add_corp_tag", data=data)

    def edit_corp_tag(self, id: str, name: Optional[str] = None, order: Optional[int] = None) -> dict:
        """
        编辑企业客户标签

        企业可通过此接口编辑客户标签/标签组的名称或次序值。

        **注意**: 修改后的标签组不能和已有的标签组重名，标签也不能和同一标签组下的其他标签重名。

        详细请查阅企业微信官方文档 `编辑企业客户标签`_ 章节。

        使用示例:

        .. code-block:: python

            from wechatpy.exceptions import WeChatClientException
            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            # 修改标签
            try:
                client.external_contact.edit_corp_tag(id="TAG_ID", name="NEW_TAG_NAME")
            except WeChatClientException as err:
                # 标签修改失败时的处理
                ...

        :param id: 标签或标签组的id列表
        :param name: 新的标签或标签组名称，最长为30个字符
        :param order: 标签/标签组的次序值。order值大的排序靠前。有效的值范围是[0, 2^32)
        :return: 创建结果（字典类型）

        .. _编辑企业客户标签: https://work.weixin.qq.com/api/doc/90000/90135/9211
            7#%E7%BC%96%E8%BE%91%E4%BC%81%E4%B8%9A%E5%AE%A2%E6%88%B7%E6%A0%87%
            E7%AD%BE

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 对于添加/删除/编辑企业客户标签接口，目前仅支持使用 `客户联系secret`_ 所获
              取的accesstoken来调用。
            - 第三方仅可读取，且应用需具有 `企业客户权限`_。

        .. warning:: 暂不支持第三方调用。
        """
        data = optionaldict(id=id, name=name, order=order)
        return self._post("externalcontact/edit_corp_tag", data=data)

    def del_corp_tag(self, tag_id: Optional[str] = None, group_id: Optional[str] = None) -> dict:
        """
        删除企业客户标签

        企业可通过此接口删除客户标签库中的标签，或删除整个标签组。

        详细请查阅企业微信官方文档 `删除企业客户标签`_ 章节。

        **参数注意事项**:

        - ``tag_id`` 和 ``group_id`` 不可同时为空。
        - 如果一个标签组下所有的标签均被删除，则标签组会被自动删除。

        使用示例:

        .. code-block:: python

            from wechatpy.exceptions import WeChatClientException
            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            try:
                client.external_contact.del_corp_tag(tag_id="TAG_ID")
            except WeChatClientException as err:
                # 标签删除失败时的处理
                ...

        :param tag_id: 标签的id列表
        :param group_id: 标签组的id列表
        :return: 删除结果（字典类型）

        .. _删除企业客户标签: https://work.weixin.qq.com/api/doc/90000/90135/9211
            7#%E5%88%A0%E9%99%A4%E4%BC%81%E4%B8%9A%E5%AE%A2%E6%88%B7%E6%A0%87%
            E7%AD%BE

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 对于添加/删除/编辑企业客户标签接口，目前仅支持使用 `客户联系secret`_ 所获
              取的accesstoken来调用。
            - 第三方仅可读取，且应用需具有 `企业客户权限`_。

        .. warning:: 暂不支持第三方调用。
        """
        data = optionaldict(tag_id=tag_id, group_id=group_id)
        return self._post("externalcontact/del_corp_tag", data=data)

    def mark_tag(
        self,
        userid: str,
        external_userid: str,
        add_tag: Optional[List[str]] = None,
        remove_tag: Optional[List[str]] = None,
    ) -> dict:
        """
        编辑客户企业标签

        企业可通过此接口为指定成员的客户添加上由 `企业统一配置的标签`_。

        详细请查阅企业微信官方文档 `编辑客户企业标签`_ 章节。

        **参数注意事项**:

        - 请确保 ``external_userid`` 是 ``userid`` 的外部联系人。
        - ``add_tag`` 和 ``remove_tag`` 不可同时为空。
        - 同一个标签组下现已支持多个标签

        使用示例:

        .. code-block:: python

            from wechatpy.exceptions import WeChatClientException
            from wechatpy.work import WeChatClient

            # 需要注意使用正确的secret，否则会导致在之后的接口调用中失败
            client = WeChatClient("corp_id", "secret_key")
            try:
                client.external_contact.mark_tag(
                    userid="USER_ID",
                    external_userid="EXT_ID",
                    add_tag=["TAG_ID_1", "TAG_ID_2"],
                )
            except WeChatClientException as err:
                # 编辑失败时的处理
                ...

        :param userid: 添加外部联系人的userid
        :param external_userid: 外部联系人userid
        :param add_tag: 要标记的标签列表
        :param remove_tag: 要移除的标签列表
        :return: 处理结果（字典类型）

        .. _企业统一配置的标签: https://work.weixin.qq.com/api/doc/90000/90135/92
            118#17298

        .. _编辑客户企业标签: https://work.weixin.qq.com/api/doc/90000/90135/92118

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方调用时，应用需具有外部联系人管理权限。
        """
        add_tag = add_tag or []
        remove_tag = remove_tag or []
        data = optionaldict(userid=userid, external_userid=external_userid, add_tag=add_tag, remove_tag=remove_tag)
        return self._post("externalcontact/mark_tag", data=data)

    def get_group_chat_list(
        self,
        limit: int,
        status_filter: int = 0,
        owner_filter: Optional[dict] = None,
        cursor: Optional[str] = None,
    ) -> dict:
        """
        获取客户群列表

        该接口用于获取配置过客户群管理的客户群列表。

        详细请查阅企业微信官方文档 `获取客户群列表`_ 章节。

        :param limit: 分页，预期请求的数据量，取值范围 1 ~ 1000
        :param status_filter: 客户群跟进状态过滤（默认为0）。
            0: 所有列表(即不过滤)
            1: 离职待继承
            2: 离职继承中
            3: 离职继承完成
        :param owner_filter: 群主过滤。
            如果不填，表示获取应用可见范围内全部群主的数据
            （但是不建议这么用，如果可见范围人数超过1000人，为了防止数据包过大，会报错 81017）
        :param cursor: 用于分页查询的游标，字符串类型，由上一次调用返回，首次调用不填
        :return: 响应数据

        .. warning::

           如果不指定 ``owner_filter``，会拉取应用可见范围内的所有群主的数据，
           但是不建议这样使用。如果可见范围内人数超过1000人，为了防止数据包过大，
           会报错 81017。此时，调用方需通过指定 ``owner_filter`` 来缩小拉取范围。

           旧版接口以 ``offset+limit`` 分页，要求 ``offset+limit`` 不能超过50000，
           该方案将废弃，请改用 ``cursor+limit`` 分页。

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户基础信息”权限
            - 对于第三方/自建应用，群主必须在应用的可见范围。

        .. _获取客户群列表: https://work.weixin.qq.com/api/doc/90000/90135/92120
        """
        data = optionaldict(status_filter=status_filter, owner_filter=owner_filter, cursor=cursor, limit=limit)
        return self._post("externalcontact/groupchat/list", data=data)

    def get_group_chat_info(self, chat_id: str) -> dict:
        """
        获取客户群详情

        通过客户群ID，获取详情。包括群名、群成员列表、群成员入群时间、入群方式。
        （客户群是由具有客户群使用权限的成员创建的外部群）

        需注意的是，如果发生群信息变动，会立即收到群变更事件，但是部分信息是异步处理，
        可能需要等一段时间调此接口才能得到最新结果

        详细请查阅企业微信官方文档 `获取客户群详情`_ 章节。

        :param chat_id: 客户群ID
        :return: 响应数据

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户基础信息”权限
            - 对于第三方/自建应用，群主必须在应用的可见范围。

        .. _获取客户群详情: https://work.weixin.qq.com/api/doc/90000/90135/92122
        """
        data = optionaldict(chat_id=chat_id)
        return self._post("externalcontact/groupchat/get", data=data)

    def add_group_welcome_template(self, template: dict, agentid: Optional[int] = None) -> dict:
        """
        添加群欢迎语素材

        企业可通过此API向企业的入群欢迎语素材库中添加素材。每个企业的入群欢迎语素材库中，
        最多容纳100个素材。

        详细请查阅企业微信官方文档 `添加群欢迎语素材`_ 章节。

        :param template: 群欢迎语素材内容，详细字段请参考微信文档
        :param agentid: 授权方安装的应用agentid。仅旧的第三方多应用套件需要填此参数
        :return: 响应数据

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户联系->配置入群欢迎语素材”权限

        .. _添加群欢迎语素材: https://work.weixin.qq.com/api/doc/90000/90135/
            92366#%E6%B7%BB%E5%8A%A0%E5%85%A5%E7%BE%A4%E6%AC%A2%E8%BF%8E%E
            8%AF%AD%E7%B4%A0%E6%9D%90

        """
        data = optionaldict()
        data.update(template)
        data["agentid"] = agentid
        return self._post("externalcontact/group_welcome_template/add", data=data)

    def update_group_welcome_template(self, template: dict, template_id: str, agentid: Optional[int] = None) -> dict:
        """
        编辑群欢迎语素材

        企业可通过此API编辑入群欢迎语素材库中的素材，且仅能够编辑调用方自己创建的入群欢迎语素材。


        详细请查阅企业微信官方文档 `编辑群欢迎语素材`_ 章节。

        :param template: 群欢迎语素材内容，详细字段请参考微信文档
        :param template_id: 欢迎语素材id
        :param agentid: 授权方安装的应用agentid。仅旧的第三方多应用套件需要填此参数
        :return: 响应数据

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户联系->配置入群欢迎语素材”权限
            - 仅可编辑本应用创建的入群欢迎语素材

        .. _添编辑群欢迎语素材: https://work.weixin.qq.com/api/doc/90000/90135/
            92366#%E7%BC%96%E8%BE%91%E5%85%A5%E7%BE%A4%E6%AC%A2%E8%BF%8E%E8%
            AF%AD%E7%B4%A0%E6%9D%90
        """
        data = optionaldict()
        data.update(template)
        data["template_id"] = template_id
        data["agentid"] = agentid
        return self._post("externalcontact/group_welcome_template/edit", data=data)

    def get_group_welcome_template(self, template_id: str) -> dict:
        """
        获取入群欢迎语素材

        企业可通过此API获取入群欢迎语素材。

        详细请查阅企业微信官方文档 `获取入群欢迎语素`_ 章节。

        :param template_id: 群欢迎语的素材id
        :return: 响应数据

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户联系->配置入群欢迎语素材”权限

        .. _获取入群欢迎语素材: https://work.weixin.qq.com/api/doc/90000/90135/
            92366#%E8%8E%B7%E5%8F%96%E5%85%A5%E7%BE%A4%E6%AC%A2%E8%BF%8E%E8%
            AF%AD%E7%B4%A0%E6%9D%90
        """
        data = optionaldict(template_id=template_id)
        return self._post("externalcontact/group_welcome_template/get", data=data)

    def del_group_welcome_template(self, template_id: str, agentid: Optional[int] = None) -> dict:
        """
        删除入群欢迎语素材

        企业可通过此API删除入群欢迎语素材，且仅能删除调用方自己创建的入群欢迎语素材。

        详细请查阅企业微信官方文档 `删除入群欢迎语素材`_ 章节。

        :param template_id: 群欢迎语的素材id
        :param agentid: 授权方安装的应用agentid。仅旧的第三方多应用套件需要填此参数
        :return: 响应数据

        .. note::
            **权限说明：**

            - 需要使用 `客户联系secret`_ 或配置到 `可调用应用`_ 列表中的自建应用secret
              来初始化 :py:class:`wechatpy.work.client.WeChatClient` 类。
            - 第三方应用需具有“企业客户权限->客户联系->配置入群欢迎语素材”权限
            - 仅可删除本应用创建的入群欢迎语素材

        .. _删除入群欢迎语素材: https://work.weixin.qq.com/api/doc/90000/90135/
            92366#%E5%88%A0%E9%99%A4%E5%85%A5%E7%BE%A4%E6%AC%A2%E8%BF%8E%E8
            %AF%AD%E7%B4%A0%E6%9D%90
        """
        data = optionaldict(template_id=template_id, agentid=agentid)
        return self._post("externalcontact/group_welcome_template/del", data=data)



# -*- coding: utf-8 -*-
import logging

from wechatpy.client import WeChatClient  # NOQA
from wechatpy.component import ComponentOAuth, WeChatComponent  # NOQA
from wechatpy.exceptions import (
    WeChatClientException,
    WeChatException,
    WeChatOAuthException,
    WeChatPayException,
)  # NOQA
from wechatpy.oauth import WeChatOAuth  # NOQA
from wechatpy.parser import parse_message  # NOQA
from wechatpy.pay import WeChatPay  # NOQA
from wechatpy.replies import create_reply  # NOQA

__version__ = "2.0.0.alpha23"
__author__ = "messense"

# Set default logging handler to avoid "No handler found" warnings.
logging.getLogger(__name__).addHandler(logging.NullHandler())
